SystemOrganization addCategory: #'Magritte-Model-Core'!
SystemOrganization addCategory: #'Magritte-Model-Models'!
SystemOrganization addCategory: #'Magritte-Model-Description'!
SystemOrganization addCategory: #'Magritte-Model-Accessor'!
SystemOrganization addCategory: #'Magritte-Model-Memento'!
SystemOrganization addCategory: #'Magritte-Model-Condition'!
SystemOrganization addCategory: #'Magritte-Model-Visitor'!
SystemOrganization addCategory: #'Magritte-Model-Exception'!
SystemOrganization addCategory: #'Magritte-Model-Utility'!

!Symbol methodsFor: '*magritte-model-testing' stamp: 'lr 3/27/2006 17:44'!
isDescriptionDefinition
	"Answer wheter the receiver is a method selector following the naming conventions of a description definition."

	^ self isDescriptionSelector
		and: [ self isUnary ]! !

!Symbol methodsFor: '*magritte-model-testing' stamp: 'lr 3/27/2006 17:42'!
isDescriptionExtension: aSelector
	"Answer wheter the receiver is a method selector following the naming conventions of a description extension to aSelector."

	^ self isDescriptionSelector
		and: [ self numArgs = 1 
		and: [ self beginsWith: aSelector ] ]! !

!Symbol methodsFor: '*magritte-model-testing' stamp: 'lr 3/27/2006 17:43'!
isDescriptionSelector
	"Answer wheter the receiver is a method selector following the naming conventions of a  description selector."

	^ self ~= #description 
		and: [ self beginsWith: #description ]! !

!Symbol methodsFor: '*magritte-model-evaluating' stamp: 'lr 3/9/2006 11:31'!
value: anObject
	^ anObject perform: self! !

!Symbol methodsFor: '*magritte-model-evaluating' stamp: 'lr 3/9/2006 11:31'!
value: anObject1 value: anObject2
	^ anObject1 perform: self with: anObject2! !

Object subclass: #MACompatibility
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Core'!

!MACompatibility commentStamp: 'lr 2/26/2006 09:50' prior: 0!
I provivide all the platform compatibility code on my classside, so that porting to different Smalltalk dialects can concentrate onto a single place.!

!MACompatibility class methodsFor: 'user-interface' stamp: 'lr 3/9/2006 11:32'!
choose: aString from: aCollection
	"Display aString to the user and let him choose any of the items in aCollection."

	| index |
	index := PopUpMenu
		withCaption: aString
		chooseFrom: (aCollection collect: [ :each | each asString ]).
	^ index isZero
		ifFalse: [ aCollection at: index ]! !

!MACompatibility class methodsFor: 'environment' stamp: 'lr 3/9/2006 11:32'!
classNamed: aString
	"Return the class named aString, nil if the class can't be found."

	^ Smalltalk classNamed: aString! !

!MACompatibility class methodsFor: 'user-interface' stamp: 'lr 3/9/2006 11:32'!
confirm: aString
	"Display aString to the user and let him confirm."

	^ PopUpMenu confirm: aString! !

!MACompatibility class methodsFor: 'user-interface' stamp: 'lr 3/9/2006 11:32'!
inform: aString
	"Display aString to the user."

	^ PopUpMenu inform: aString! !

!MACompatibility class methodsFor: 'user-interface' stamp: 'lr 3/9/2006 11:32'!
openWorkspace: aContentsString titled: aTitleString
	"Open a new wokspace with the contents aContentsString and the title aTitleString."

	Workspace new
		contents: aContentsString;
		openLabel: aTitleString! !

!MACompatibility class methodsFor: 'environment' stamp: 'lr 3/9/2006 11:32'!
referenceStream: aReadWriteStream
	"Return a stream instance to operate on aReadWriteStream being able to serialize and deserialize objects by sending #nextPut: and #next. Squeak: The implementation of ReferenceStream doesn't work well together with the default WriteStream implementaiton, therefor we have to change it on the fly."

	^ ReferenceStream on: (aReadWriteStream class = WriteStream
		ifTrue: [ RWBinaryOrTextStream on: aReadWriteStream contents ]
		ifFalse: [ aReadWriteStream ])! !

!MACompatibility class methodsFor: 'user-interface' stamp: 'lr 3/9/2006 11:32'!
request: aString default: anObject
	"Display aString and requests an input from the user, whereas anObject is the default value."

	^ FillInTheBlankMorph
		request: aString
		initialAnswer: anObject
		centerAt: ActiveHand cursorPoint
		inWorld: ActiveWorld
		onCancelReturn: nil! !

!MACompatibility class methodsFor: 'environment' stamp: 'lr 3/9/2006 11:32'!
uuid
	"Answer a random object that is extremly likely to be unique over space and time."

	^ UUID new! !

Object subclass: #MADescriptionBuilder
	instanceVariableNames: 'cache'
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'Magritte-Model-Utility'!

!MADescriptionBuilder class methodsFor: 'accessing' stamp: 'lr 3/27/2006 13:31'!
default
	^ Default! !

!MADescriptionBuilder class methodsFor: 'accessing' stamp: 'lr 3/27/2006 13:31'!
default: aBuilder
	Default := aBuilder! !

!MADescriptionBuilder class methodsFor: 'building' stamp: 'lr 3/27/2006 13:29'!
for: anObject
	^ self default for: anObject! !

!MADescriptionBuilder class methodsFor: 'initialization' stamp: 'lr 3/27/2006 17:21'!
initialize
	self default: MANamedBuilder new! !

!MADescriptionBuilder methodsFor: 'actions' stamp: 'lr 3/27/2006 13:54'!
build: anObject
	self subclassResponsibility! !

!MADescriptionBuilder methodsFor: 'initialization' stamp: 'lr 3/27/2006 13:55'!
finalize
	super finalize.
	self flush; unregister! !

!MADescriptionBuilder methodsFor: 'actions' stamp: 'lr 3/27/2006 13:41'!
flush
	cache := IdentityDictionary new! !

!MADescriptionBuilder methodsFor: 'accessing' stamp: 'lr 3/27/2006 13:58'!
for: anObject
	^ cache at: anObject ifAbsentPut: [ self build: anObject ]! !

!MADescriptionBuilder methodsFor: 'initialization' stamp: 'lr 3/27/2006 13:34'!
initialize
	super initialize.
	self flush; register! !

!MADescriptionBuilder methodsFor: 'events' stamp: 'lr 3/27/2006 13:55'!
modified: anEvent
	self flush! !

!MADescriptionBuilder methodsFor: 'actions' stamp: 'lr 3/27/2006 13:36'!
register
	SystemChangeNotifier uniqueInstance
		notify: self ofSystemChangesOfItem: #method change: #Added using: #modified:;
		notify: self ofSystemChangesOfItem: #method change: #Modified using: #modified:;
		notify: self ofSystemChangesOfItem: #method change: #Removed using: #modified:! !

!MADescriptionBuilder methodsFor: 'actions' stamp: 'lr 3/27/2006 14:45'!
unregister
	SystemChangeNotifier uniqueInstance
		noMoreNotificationsFor: self! !

MADescriptionBuilder subclass: #MANamedBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Utility'!

!MANamedBuilder methodsFor: 'actions' stamp: 'lr 3/27/2006 17:42'!
build: anObject
	| selectors description |
	selectors := anObject class allSelectors 
		select: #isDescriptionSelector.
	^ (selectors select: #isDescriptionDefinition)
		inject: anObject defaultContainer
		into: [ :result :each |
			description := self build: anObject for: each in: selectors.
			description isDescription
				ifTrue: [ result add: description ].
			result ]! !

!MANamedBuilder methodsFor: 'actions' stamp: 'lr 3/27/2006 17:34'!
build: anObject for: aSelector in: aCollection
	^ (aCollection select: [ :each | each isDescriptionExtension: aSelector ])
		inject: (anObject perform: aSelector)
		into: [ :result :each | anObject perform: each with: result ]! !

MADescriptionBuilder subclass: #MAPragmaBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Utility'!

!MAPragmaBuilder methodsFor: 'actions' stamp: 'lr 3/27/2006 15:54'!
build: anObject
	| selectors |
	selectors := Set new.
	^ (Pragma allNamed: #description from: anObject class to: Object class)
		inject: anObject defaultContainer
		into: [ :container :pragma |
			(selectors includes: pragma selector) ifFalse: [
				selectors add: pragma selector.
				container add: (self build: anObject from: pragma) ].
			container ]! !

!MAPragmaBuilder methodsFor: 'actions' stamp: 'lr 3/27/2006 16:22'!
build: anObject from: aPragma
	^ (Pragma allNamed: #description: from: anObject class to: Object class) reverse
		inject: (anObject perform: aPragma selector)
		into: [ :description :pragma |
			aPragma selector = pragma arguments first
				ifFalse: [ description ]
				ifTrue: [ anObject perform: pragma selector with: description ] ]! !

Object subclass: #MADistribution
	instanceVariableNames: 'zip username password preamble postscript prepared'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Core'!

!MADistribution commentStamp: 'lr 2/26/2006 09:53' prior: 0!
I am responsible to build a distribution and to publish a complete package on SqueakMap. All my settings, such as what packages it depends on, are defined on the class-side and can be overridden to create different distributions basing on Magritte.!

!MADistribution class methodsFor: 'utilities' stamp: 'lr 3/9/2006 11:33'!
cleanCategories: aPackage
	aPackage classesAndMetaClasses do: [ :each |
		each organization
			removeEmptyCategories;
			sortCategories ]! !

!MADistribution class methodsFor: 'utilities' stamp: 'lr 3/21/2006 20:03'!
cleanSource: aPackage
	| input output line |
	aPackage methods do: [ :each |
		input := each sourceCode asString
			withBlanksTrimmed readStream.
		output := String streamContents: [ :stream |
			[ input atEnd ] whileFalse: [
				line := input nextLine.
				stream nextPutAll: (line
					copyFrom: 1 to: (line
						findLast: [ :c | c isSeparator not ])).
				stream nextPut: Character cr ] ].
		output := output allButLast.
		(output last = $. and: [ (output at: output size - 1) ~= $$ ])
			ifTrue: [ output := output allButLast ].
		each sourceCode ~= output
			ifTrue: [ each actualClass compile: output ] ]! !

!MADistribution class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
license
	"Return a string with the license of the package. This string shall not be removed or altered in any case."

	^ 'The MIT License

Copyright (c) 2003-2006 Lukas Renggli, renggli at iam.unibe.ch
Copyright (c) 2003-2006 Software Composition Group

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.'! !

!MADistribution class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
package
	"The SqueakMap package name."

	^ 'Magritte'! !

!MADistribution class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
packages
	"Return a collection of package names that should be distributed and eventually installed."

	^ #( ( 'DynamicBindings' '(Smalltalk includesKey: #WAComponent) or: [ PasteUpMorph confirm: ''Would you like to install Seaside?'' ]' )
		 ( 'KomServices' 'Smalltalk includesKey: #DynamicBindings' )
		 ( 'KomHttpServer' 'Smalltalk includesKey: #TcpService' )
		 ( 'Seaside2' 'Smalltalk includesKey: #HttpService' )
		 ( 'Scriptaculous' 'Smalltalk includesKey: #WAComponent' )

		 ( 'Magritte-Model' )
		 ( 'Magritte-Tests' 'Smalltalk includesKey: #TestCase' )
		 ( 'Magritte-Morph' 'Smalltalk includesKey: #Morph' )
		 ( 'Magritte-Seaside' 'Smalltalk includesKey: #WAComponent' ) )! !

!MADistribution class methodsFor: 'execution' stamp: 'lr 3/9/2006 11:33'!
postload
	"This method will be automatically called after successful loading the distribution."

	MACompatibility openWorkspace: self license titled: 'License'.
	self registerRepository: 'http://mc.lukas-renggli.ch/magritte'! !

!MADistribution class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
postscript
	^ self name , ' postload.'! !

!MADistribution class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
preamble
	^ String new! !

!MADistribution class methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:33'!
productName
	^ String streamContents: [ :stream |
		stream
			nextPutAll: self package; space;
			nextPutAll: self versionNumber ]! !

!MADistribution class methodsFor: 'private' stamp: 'lr 3/9/2006 11:33'!
registerRepository: aString
	| repositoryGroup httpRespository |
	repositoryGroup := Smalltalk classNamed: #MCRepositoryGroup.
	httpRespository := Smalltalk classNamed: #MCHttpRepository.
	(repositoryGroup notNil and: [ httpRespository notNil ]) ifTrue: [
		repositoryGroup default
			addRepository: ((httpRespository
				location: aString
				user: String new
				password: String new)
					creationTemplate: 'MCHttpRepository
	location: ' , aString printString , '
	user: ''''
	password: ''''';
				yourself) ]! !

!MADistribution class methodsFor: 'accessing' stamp: 'lr 3/21/2006 19:16'!
version
	"The first item is the product name, which should only contain lowercase letters. Then the version numbers are following, that are made up of three numbers. An optimal suffix alpha, beta, pre or rc follows. External packages or extensions might want to call this method to check the version required to run."

	^ #( magritte 1 0 8 )! !

!MADistribution class methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:33'!
versionName
	^ self version first asString! !

!MADistribution class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
versionNote
	"This is a string to be used as release-note in SqueakMap."

	^ String new! !

!MADistribution class methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:33'!
versionNumber
	^ String streamContents: [ :output |
		self version allButFirst readStream in: [ :input |
			[ input atEnd or: [ input peek isString ] ]
				whileFalse: [ output print: input next; nextPut: $. ].
			output skip: -1.
			input atEnd
				ifFalse: [ output nextPut: $-; nextPutAll: input next ] ] ]! !

!MADistribution class methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:33'!
versionString
	^ String streamContents: [ :stream |
		stream
			nextPutAll: self versionName; nextPut: $-;
			nextPutAll: self versionNumber ]! !

!MADistribution methodsFor: 'adding' stamp: 'lr 3/9/2006 11:32'!
addPackage: aPackage
	(self preparedPackages includes: aPackage)
		ifTrue: [ ^ self ].
	self
		addPackage: aPackage
		workingCopy: (MCWorkingCopy registry
			at: aPackage)! !

!MADistribution methodsFor: 'adding' stamp: 'lr 3/9/2006 11:32'!
addPackage: aPackage version: aVersion
	| condition |
	self zip
		addString: (ByteArray streamContents: [ :stream |
			aVersion fileOutOn: stream ])
		as: aVersion fileName.
	self addPreamble: (String streamContents: [ :stream |
		condition := self class packages
			detect: [ :each | aPackage name = each first ]
			ifNone: [ Array with: aPackage name ].
		condition size > 1
			ifFalse: [ condition := condition copyWith: 'true' ].
		stream
			nextPut: $(; nextPutAll: condition second; nextPutAll: ') ifTrue: [ ';
			nextPutAll: 'self fileInMonticelloZipVersionNamed: ';
			print: aVersion fileName; nextPutAll: ' ].' ]).
	self preparedPackages add: aPackage! !

!MADistribution methodsFor: 'adding' stamp: 'lr 3/9/2006 11:32'!
addPackage: aPackage workingCopy: aWorkingCopy
	aWorkingCopy modified
		ifTrue: [ self error: aPackage name , ' needs to be saved first.' ].
	aWorkingCopy requiredPackages
		do: [ :each | self addPackage: each ].
	self
		addPackage: aPackage
		version: (MCVersion
			package: aPackage
			info: (aWorkingCopy ancestry
				infoWithName: aWorkingCopy ancestors first name
				message: self versionString)
			snapshot: aPackage snapshot)! !

!MADistribution methodsFor: 'adding' stamp: 'lr 3/9/2006 11:32'!
addPostscript: aString
	postscript := self postscriptString copyWithAll: aString , String cr! !

!MADistribution methodsFor: 'adding' stamp: 'lr 3/9/2006 11:32'!
addPreamble: aString
	preamble := self preambleString copyWithAll: aString , String cr! !

!MADistribution methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:32'!
basePackages
	^ self class packages collect: [ :each | MCPackage named: each first ]! !

!MADistribution methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
checkResult: aString
	| result |
	result := #( 'HTTP/1.1 201 ' 'HTTP/1.1 200 ' 'HTTP/1.0 201 ' 'HTTP/1.0 200 ')
		anySatisfy: [ :each | aString beginsWith: each ].
	result ifFalse: [ self error: aString ]! !

!MADistribution methodsFor: 'building' stamp: 'lr 3/9/2006 11:32'!
dump
	"Dump the sar-archive to the file-system."

	self zip writeToFileNamed: self filename! !

!MADistribution methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
filename
	^ self versionString , '.sar'! !

!MADistribution methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:32'!
initialize
	super initialize.
	self basePackages
		do: [ :each | self addPackage: each ]
		displayingProgress: self versionString.
	(self zip addString: self preambleString as: 'install/preamble')
		desiredCompressionLevel: 9.
	(self zip addString: self postscriptString as: 'install/postscript')
		desiredCompressionLevel: 9! !

!MADistribution methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
password
	^ password ifNil: [ password := MACompatibility request: 'SqueakMap Password:' default: '' ]! !

!MADistribution methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:32'!
postscriptString
	^ postscript ifNil: [ postscript := self class postscript ]! !

!MADistribution methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:32'!
preambleString
	^ preamble ifNil: [ preamble := self class preamble ]! !

!MADistribution methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:32'!
preparedPackages
	^ prepared ifNil: [ prepared := Set new ]! !

!MADistribution methodsFor: 'building' stamp: 'lr 3/9/2006 11:32'!
publish
	"Publish the receiver as a new release on SqueakMap."

	| result |
	result := HTTPSocket
		httpPost: self squeakMapUrl , 'packagebyname/' , self class package , '/newrelease'
		args: (Array
			with: 'version' -> (Array with: self versionNumber)
			with: 'note' -> (Array with: self versionNote)
			with: 'downloadURL' -> (Array with: self upload)
			with: 'name' -> (Array with: self versionName)
			with: 'summary' -> (Array with: self versionString))
		user: self username
		passwd: self password.
	(result contents includesSubString: self versionNumber)
		ifFalse: [ self error: result contents ]! !

!MADistribution methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
squeakMapUrl
	^ 'http://map1.squeakfoundation.org/sm/'! !

!MADistribution methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
upload
	| result stream |
	result := HTTPSocket
		httpPut: (ByteArray streamContents: [ :s | self zip writeTo: s ])
		to: self squeakMapUrl , 'upload/' , self filename
		user: self username passwd: self password.
	self checkResult: result.
	stream := result readStream.
	stream upToAll: 'http://'.
	^ 'http://' , stream upToEnd! !

!MADistribution methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
username
	^ username ifNil: [ username := MACompatibility request: 'SqueakMap Username:' default: 'lr' ]! !

!MADistribution methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:32'!
versionName
	^ self class versionName! !

!MADistribution methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:32'!
versionNote
	^ self class versionNote! !

!MADistribution methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:32'!
versionNumber
	^ self class versionNumber! !

!MADistribution methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:32'!
versionString
	^ self class versionString! !

!MADistribution methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
zip
	^ zip ifNil: [ zip := ZipArchive new ]! !

Object subclass: #MAFileModel
	instanceVariableNames: 'contents mimetype filename'
	classVariableNames: 'MimeTypes'
	poolDictionaries: ''
	category: 'Magritte-Model-Models'!

!MAFileModel commentStamp: 'lr 3/1/2006 14:26' prior: 0!
I am representing a file with filename, mimetype and contents within the Magritte framework.!

!MAFileModel class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:33'!
defaultMimeType
	^ 'application/octet-stream'! !

!MAFileModel class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:33'!
defaultMimeTypes
	^ #( 'ai' 'application/postscript' 'aif' 'audio/x-aiff' 'aifc' 'audio/x-aiff' 'aiff' 'audio/x-aiff' 'asc' 'text/plain' 'au' 'audio/basic' 'avi' 'video/x-msvideo' 'bcpio' 'application/x-bcpio' 'bin' 'application/octet-stream' 'c' 'text/plain' 'cc' 'text/plain' 'ccad' 'application/clariscad' 'cdf' 'application/x-netcdf' 'class' 'application/octet-stream' 'cpio' 'application/x-cpio' 'cpt' 'application/mac-compactpro' 'csh' 'application/x-csh' 'css' 'text/css' 'dcr' 'application/x-director' 'dir' 'application/x-director' 'dms' 'application/octet-stream' 'doc' 'application/msword' 'drw' 'application/drafting' 'dvi' 'application/x-dvi' 'dwg' 'application/acad' 'dxf' 'application/dxf' 'dxr' 'application/x-director' 'eps' 'application/postscript' 'etx' 'text/x-setext' 'exe' 'application/octet-stream' 'ez' 'application/andrew-inset' 'f' 'text/plain' 'f90' 'text/plain' 'fli' 'video/x-fli' 'gif' 'image/gif' 'gtar' 'application/x-gtar' 'gz' 'application/x-gzip' 'h' 'text/plain' 'hdf' 'application/x-hdf' 'hh' 'text/plain' 'hqx' 'application/mac-binhex40' 'htm' 'text/html' 'html' 'text/html' 'ice' 'x-conference/x-cooltalk' 'ief' 'image/ief' 'iges' 'model/iges' 'igs' 'model/iges' 'ips' 'application/x-ipscript' 'ipx' 'application/x-ipix' 'jpe' 'image/jpeg' 'jpeg' 'image/jpeg' 'jpg' 'image/jpeg' 'js' 'application/x-javascript' 'kar' 'audio/midi' 'latex' 'application/x-latex' 'lha' 'application/octet-stream' 'lsp' 'application/x-lisp' 'lzh' 'application/octet-stream' 'm' 'text/plain' 'man' 'application/x-troff-man' 'me' 'application/x-troff-me' 'mesh' 'model/mesh' 'mid' 'audio/midi' 'midi' 'audio/midi' 'mif' 'application/vnd.mif' 'mime' 'www/mime' 'mov' 'video/quicktime' 'movie' 'video/x-sgi-movie' 'mp2' 'audio/mpeg' 'mp3' 'audio/mpeg' 'mpe' 'video/mpeg' 'mpeg' 'video/mpeg' 'mpg' 'video/mpeg' 'mpga' 'audio/mpeg' 'ms' 'application/x-troff-ms' 'msh' 'model/mesh' 'nc' 'application/x-netcdf' 'oda' 'application/oda' 'pbm' 'image/x-portable-bitmap' 'pdb' 'chemical/x-pdb' 'pdf' 'application/pdf' 'pgm' 'image/x-portable-graymap' 'pgn' 'application/x-chess-pgn' 'png' 'image/png' 'pnm' 'image/x-portable-anymap' 'pot' 'application/mspowerpoint' 'ppm' 'image/x-portable-pixmap' 'pps' 'application/mspowerpoint' 'ppt' 'application/mspowerpoint' 'ppz' 'application/mspowerpoint' 'pre' 'application/x-freelance' 'prt' 'application/pro_eng' 'ps' 'application/postscript' 'qt' 'video/quicktime' 'ra' 'audio/x-realaudio' 'ram' 'audio/x-pn-realaudio' 'ras' 'image/cmu-raster' 'rgb' 'image/x-rgb' 'rm' 'audio/x-pn-realaudio' 'roff' 'application/x-troff' 'rpm' 'audio/x-pn-realaudio-plugin' 'rtf' 'text/rtf' 'rtx' 'text/richtext' 'scm' 'application/x-lotusscreencam' 'set' 'application/set' 'sgm' 'text/sgml' 'sgml' 'text/sgml' 'sh' 'application/x-sh' 'shar' 'application/x-shar' 'silo' 'model/mesh' 'sit' 'application/x-stuffit' 'skd' 'application/x-koan' 'skm' 'application/x-koan' 'skp' 'application/x-koan' 'skt' 'application/x-koan' 'smi' 'application/smil' 'smil' 'application/smil' 'snd' 'audio/basic' 'sol' 'application/solids' 'spl' 'application/x-futuresplash' 'src' 'application/x-wais-source' 'step' 'application/STEP' 'stl' 'application/SLA' 'stp' 'application/STEP' 'sv4cpio' 'application/x-sv4cpio' 'sv4crc' 'application/x-sv4crc' 'swf' 'application/x-shockwave-flash' 't' 'application/x-troff' 'tar' 'application/x-tar' 'tcl' 'application/x-tcl' 'tex' 'application/x-tex' 'texi' 'application/x-texinfo' 'texinfo' 'application/x-texinfo' 'tif' 'image/tiff' 'tiff' 'image/tiff' 'tr' 'application/x-troff' 'tsi' 'audio/TSP-audio' 'tsp' 'application/dsptype' 'tsv' 'text/tab-separated-values' 'txt' 'text/plain' 'unv' 'application/i-deas' 'ustar' 'application/x-ustar' 'vcd' 'application/x-cdlink' 'vda' 'application/vda' 'viv' 'video/vnd.vivo' 'vivo' 'video/vnd.vivo' 'vrml' 'model/vrml' 'wav' 'audio/x-wav' 'wrl' 'model/vrml' 'xbm' 'image/x-xbitmap' 'xlc' 'application/vnd.ms-excel' 'xll' 'application/vnd.ms-excel' 'xlm' 'application/vnd.ms-excel' 'xls' 'application/vnd.ms-excel' 'xlw' 'application/vnd.ms-excel' 'xml' 'text/xml' 'xpm' 'image/x-xpixmap' 'xwd' 'image/x-xwindowdump' 'xyz' 'chemical/x-pdb' 'zip' 'application/zip' )! !

!MAFileModel class methodsFor: 'class initialization' stamp: 'lr 3/9/2006 11:33'!
initialize
	MimeTypes := Dictionary new.
	1 to: self defaultMimeTypes size by: 2 do: [ :index |
		MimeTypes
			at: (self defaultMimeTypes at: index)
			put: (self defaultMimeTypes at: index + 1) ]! !

!MAFileModel class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
mimetypeFor: aString
	^ self mimetypes at: aString ifAbsent: [ self defaultMimeType ]! !

!MAFileModel class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
mimetypes
	^ MimeTypes! !

!MAFileModel methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
contents
	^ contents ifNil: [ ByteArray new ]! !

!MAFileModel methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
contents: aByteArray
	contents := aByteArray! !

!MAFileModel methodsFor: 'querying' stamp: 'lr 3/21/2006 19:56'!
extension
	"Answer the file-extension."

	^ self filename copyAfterLast: $.! !

!MAFileModel methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
filename
	^ filename ifNil: [ String new ]! !

!MAFileModel methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
filename: aString
	filename := aString! !

!MAFileModel methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:32'!
hash
	^ self filename hash bitXor: self mimetype hash! !

!MAFileModel methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isApplication
	"Return true if the mimetype of the receiver is application-data. This message will match types like: application/postscript, application/zip, application/pdf, etc."

	^ self maintype = 'application'! !

!MAFileModel methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isAudio
	"Return true if the mimetype of the receiver is audio-data. This message will match types like: audio/basic, audio/tone, audio/mpeg, etc."

	^ self maintype = 'audio'! !

!MAFileModel methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isEmpty
	^ self size isZero! !

!MAFileModel methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isImage
	"Return true if the mimetype of the receiver is image-data. This message will match types like: image/jpeg, image/gif, image/png, image/tiff, etc."

	^ self maintype = 'image'! !

!MAFileModel methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isText
	"Return true if the mimetype of the receiver is text-data. This message will match types like: text/plain, text/html, text/sgml, text/css, text/xml, text/richtext, etc."

	^ self maintype = 'text'! !

!MAFileModel methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isVideo
	"Return true if the mimetype of the receiver is video-data. This message will match types like: video/mpeg, video/quicktime, etc."

	^ self maintype = 'video'! !

!MAFileModel methodsFor: 'querying' stamp: 'lr 3/9/2006 11:32'!
maintype
	"Answer the first part of the mime-type."

	^ self mimetype copyUpTo: $/! !

!MAFileModel methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
mimetype
	^ mimetype ifNil: [ self class mimetypeFor: self extension ]! !

!MAFileModel methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
mimetype: aString
	mimetype := aString! !

!MAFileModel methodsFor: 'querying' stamp: 'lr 3/9/2006 11:32'!
size
	"Answer the size of the file."

	^ self contents size! !

!MAFileModel methodsFor: 'querying' stamp: 'lr 3/9/2006 11:32'!
subtype
	"Answer the second part of the mime-type."

	^ self mimetype copyAfter: $/! !

!MAFileModel methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:32'!
= anObject
	^ self species = anObject species
		and: [ self filename = anObject filename ]
		and: [ self mimetype = anObject mimetype ]
		and: [ self contents = anObject contents ]! !

Object subclass: #MAObject
	instanceVariableNames: 'properties'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Core'!

!MAObject commentStamp: 'lr 2/26/2006 09:54' prior: 0!
I provide functionality available to all Magritte objects. I implement a dictionary of properties, so that extensions can easily store additional data efficiently.!

MAObject subclass: #MAAccessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Accessor'!

!MAAccessor commentStamp: 'lr 2/12/2006 20:01' prior: 0!
I am the abstract superclass to all accessor strategies. Accessors are used to implement different ways of accessing (reading and writing) data from instances using a common protocol: data can be uniformly read and written using <code>#readFrom:</code> respectively <code>write:to:</code>.!

!MAAccessor methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
canRead: aModel
	"Test if aModel can be read."

	^ true! !

!MAAccessor methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
canWrite: aModel
	"Test if aModel can be written."

	^ true! !

!MAAccessor methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAccessor
	^ true! !

!MAAccessor methodsFor: 'printing' stamp: 'lr 3/9/2006 11:31'!
printOn: aStream
	self storeOn: aStream! !

!MAAccessor methodsFor: 'model' stamp: 'lr 3/9/2006 11:31'!
read: aModel
	"Read from aModel using the access-strategy of the receiver."

	self subclassResponsibility! !

!MAAccessor methodsFor: 'printing' stamp: 'lr 3/9/2006 11:31'!
storeOn: aStream
	aStream nextPutAll: self class name; nextPutAll: ' new'! !

!MAAccessor methodsFor: 'model' stamp: 'lr 3/9/2006 11:31'!
write: anObject to: aModel
	"Write anObject to aModel using the access-strategy of the receiver."

	self subclassResponsibility! !

MAAccessor subclass: #MABlockAccessor
	instanceVariableNames: 'readBlock writeBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Accessor'!

!MABlockAccessor commentStamp: 'lr 2/12/2006 20:04' prior: 0!
I am an access strategy defined by two block-closures. The read-block expects the model as its first argument and is used to retrieve a value. The write-block expects the model as its first and the value as its second argument and is used to write a value to the model.!

!MABlockAccessor class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isAbstract
	^ false! !

!MABlockAccessor class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:32'!
read: aReadBlock write: aWriteBlock
	^ self new
		readBlock: aReadBlock;
		writeBlock: aWriteBlock;
		yourself! !

!MABlockAccessor methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
canRead: aModel
	^ (super canRead: aModel)
		and: [ self readBlock notNil ]! !

!MABlockAccessor methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
canWrite: aModel
	^ (super canWrite: aModel)
		and: [ self writeBlock notNil ]! !

!MABlockAccessor methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:31'!
hash
	^ super hash bitXor: (self readBlock hash bitXor: self writeBlock hash)! !

!MABlockAccessor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
readBlock
	^ readBlock! !

!MABlockAccessor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
readBlock: aBlock
	readBlock := aBlock! !

!MABlockAccessor methodsFor: 'model' stamp: 'lr 3/9/2006 11:31'!
read: aModel
	^ self readBlock value: aModel! !

!MABlockAccessor methodsFor: 'printing' stamp: 'lr 3/9/2006 11:31'!
storeBlock: aBlock on: aStream
	aStream nextPutAll: aBlock decompile asString allButFirst allButLast! !

!MABlockAccessor methodsFor: 'printing' stamp: 'lr 3/9/2006 11:31'!
storeOn: aStream
	aStream nextPut: $(; nextPutAll: self class name; nextPutAll: ' read: '.
	self storeBlock: self readBlock on: aStream.
	aStream nextPutAll: ' write: '.
	self storeBlock: self writeBlock on: aStream.
	aStream nextPut: $)! !

!MABlockAccessor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
writeBlock
	^ writeBlock! !

!MABlockAccessor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
writeBlock: aBlock
	writeBlock := aBlock! !

!MABlockAccessor methodsFor: 'model' stamp: 'lr 3/9/2006 11:31'!
write: anObject to: aModel
	self writeBlock value: aModel value: anObject! !

!MABlockAccessor methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:31'!
= anObject
	^ super = anObject
		and: [ self readBlock = anObject readBlock ]
		and: [ self writeBlock = anObject writeBlock ]! !

MAAccessor subclass: #MACascadeAccessor
	instanceVariableNames: 'accessor next'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Accessor'!

!MACascadeAccessor commentStamp: 'lr 2/12/2006 20:08' prior: 0!
I am an access strategy used to chain two access strategies. To read and write a value the <code>accessor</code> is perfromed on the given model and the result is used to passed into the <code>next</code> accessor.!

!MACascadeAccessor class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:32'!
accessor: anAccessor next: aNextAccessor
	^ self new
		accessor: anAccessor;
		next: aNextAccessor;
		yourself! !

!MACascadeAccessor class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isAbstract
	^ false! !

!MACascadeAccessor class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:32'!
selectors: anArray
	^ anArray allButLast reverse
		inject: (MASelectorAccessor selector: anArray last)
		into: [ :next :accessor |
			self
				accessor: (MASelectorAccessor selector: accessor)
				next: next ]! !

!MACascadeAccessor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
accessor
	^ accessor! !

!MACascadeAccessor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
accessor: anAccessor
	accessor := anAccessor! !

!MACascadeAccessor methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
canRead: aModel
	^ (super canRead: aModel)
		and: [ self accessor canRead: aModel ]
		and: [ self next canRead: (self accessor read: aModel) ]! !

!MACascadeAccessor methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
canWrite: aModel
	^ (super canWrite: aModel)
		and: [ self accessor canRead: aModel ]
		and: [ self next canWrite: (self accessor read: aModel) ]! !

!MACascadeAccessor methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:31'!
hash
	^ super hash bitXor: (self accessor hash bitXor: self next hash)! !

!MACascadeAccessor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
next
	^ next! !

!MACascadeAccessor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
next: anAccessor
	next := anAccessor! !

!MACascadeAccessor methodsFor: 'copying' stamp: 'lr 3/9/2006 11:31'!
postCopy
	super postCopy.
	self accessor: self accessor copy.
	self next: self next copy! !

!MACascadeAccessor methodsFor: 'model' stamp: 'lr 3/9/2006 11:31'!
read: aModel
	^ self next read: (self accessor read: aModel)! !

!MACascadeAccessor methodsFor: 'printing' stamp: 'lr 3/9/2006 11:31'!
storeOn: aStream
	aStream nextPut: $(; nextPutAll: self class name; nextPutAll: ' accessor: '; store: self accessor; nextPutAll: ' next: '; store: self next; nextPut: $)! !

!MACascadeAccessor methodsFor: 'model' stamp: 'lr 3/9/2006 11:31'!
write: anObject to: aModel
	self next write: anObject to: (self accessor read: aModel)! !

!MACascadeAccessor methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:31'!
= anObject
	^ super = anObject
		and: [ self accessor = anObject accessor ]
		and: [ self next = anObject next ]! !

MAAccessor subclass: #MAContainerAccessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Accessor'!

!MAContainerAccessor commentStamp: 'lr 2/12/2006 20:09' prior: 0!
I am a read-only access strategy and I answer the model itself when being read.!

!MAContainerAccessor class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isAbstract
	^ false! !

!MAContainerAccessor methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
canWrite: aModel
	^ false! !

!MAContainerAccessor methodsFor: 'model' stamp: 'lr 3/9/2006 11:32'!
read: aModel
	^ aModel! !

!MAContainerAccessor methodsFor: 'model' stamp: 'lr 3/9/2006 11:32'!
write: anObject to: aModel
	self error: 'Not supposed to write to ' , aModel asString , '.'! !

MAAccessor subclass: #MADictionaryAccessor
	instanceVariableNames: 'key'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Accessor'!

!MADictionaryAccessor commentStamp: 'lr 2/12/2006 20:11' prior: 0!
I am an access strategy to be used on dictionaries. I use my <code>key</code> to read from and write to indexed collections.!

!MADictionaryAccessor class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isAbstract
	^ false! !

!MADictionaryAccessor class methodsFor: 'instance-creation' stamp: 'lr 3/9/2006 11:32'!
key: aSymbol
	^ self new
		key: aSymbol;
		yourself! !

!MADictionaryAccessor methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
canRead: aModel
	^ (super canRead: aModel)
		and: [ aModel includesKey: self key ]! !

!MADictionaryAccessor methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:32'!
hash
	^ super hash bitXor: self key hash! !

!MADictionaryAccessor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
key
	^ key! !

!MADictionaryAccessor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
key: aKey
	key := aKey! !

!MADictionaryAccessor methodsFor: 'model' stamp: 'lr 3/9/2006 11:32'!
read: aModel
	^ aModel at: self key! !

!MADictionaryAccessor methodsFor: 'printing' stamp: 'lr 3/9/2006 11:32'!
storeOn: aStream
	aStream nextPut: $(; nextPutAll: self class name; nextPutAll: ' key: '; store: self key; nextPut: $)! !

!MADictionaryAccessor methodsFor: 'model' stamp: 'lr 3/9/2006 11:32'!
write: anObject to: aModel
	aModel at: self key put: anObject! !

!MADictionaryAccessor methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:32'!
= anObject
	^ super = anObject
		and: [ self key = anObject key ]! !

MAAccessor subclass: #MANullAccessor
	instanceVariableNames: 'uuid'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Accessor'!

!MANullAccessor commentStamp: 'lr 2/12/2006 20:15' prior: 0!
I am a null access strategy and I should be neighter read nor written. I am still comparable to other strategies by holding onto a unique-identifier.!

!MANullAccessor class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MANullAccessor class methodsFor: 'instance-creation' stamp: 'lr 3/9/2006 11:33'!
new
	^ self uuid: MACompatibility uuid! !

!MANullAccessor class methodsFor: 'instance-creation' stamp: 'lr 3/9/2006 11:33'!
uuid: anUUID
	^ self basicNew
		uuid: anUUID;
		yourself! !

!MANullAccessor methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
canRead: aModel
	^ false! !

!MANullAccessor methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
canWrite: aModel
	^ false! !

!MANullAccessor methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:32'!
hash
	^ super hash bitXor: self uuid hash! !

!MANullAccessor methodsFor: 'model' stamp: 'lr 3/9/2006 11:32'!
read: aModel
	self shouldNotImplement! !

!MANullAccessor methodsFor: 'printing' stamp: 'lr 3/9/2006 11:32'!
storeOn: aStream
	aStream nextPut: $(; nextPutAll: self class name; nextPutAll: ' uuid: '; store: self uuid; nextPut: $)! !

!MANullAccessor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
uuid
	^ uuid! !

!MANullAccessor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
uuid: anObject
	uuid := anObject! !

!MANullAccessor methodsFor: 'model' stamp: 'lr 3/9/2006 11:32'!
write: anObject to: aModel
	self shouldNotImplement! !

!MANullAccessor methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:32'!
= anObject
	^ super = anObject
		and: [ self uuid = anObject uuid ]! !

MAAccessor subclass: #MASelectorAccessor
	instanceVariableNames: 'readSelector writeSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Accessor'!

!MASelectorAccessor commentStamp: 'lr 2/12/2006 20:22' prior: 0!
I am a the most common access strategy defined by a read- and a write-selector. I am mostly used together with standard getters and setters as usually defined by the accessing protocol. If there is only a read-selector specified, the write selector will be deduced automatically by adding a colon to the read-selector.!

MASelectorAccessor subclass: #MAAutoSelectorAccessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Accessor'!

!MAAutoSelectorAccessor commentStamp: 'lr 2/12/2006 20:25' prior: 0!
I am very similar to my super-class <code>*MASelectorAccessor*</code>, however I do create instance-variables and accessor methods automatically if necessary. I am especially useful for prototyping. I never change existing accessor methods.!

!MAAutoSelectorAccessor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
categoryName
	^ #'accessing-generated'! !

!MAAutoSelectorAccessor methodsFor: 'private' stamp: 'lr 3/9/2006 11:31'!
createReadAccessor: aClass
	(aClass selectors includes: self readSelector)
		ifTrue: [ ^ self ].
	aClass
		compile: (String streamContents: [ :stream |
			stream nextPutAll: self readSelector; cr.
			stream tab; nextPutAll: '^ '; nextPutAll: self readSelector ])
		classified: self categoryName! !

!MAAutoSelectorAccessor methodsFor: 'private' stamp: 'lr 3/9/2006 11:31'!
createVariable: aClass
	(aClass allInstVarNames includes: self readSelector)
		ifTrue: [ ^ self ].
	aClass addInstVarName: self readSelector! !

!MAAutoSelectorAccessor methodsFor: 'private' stamp: 'lr 3/9/2006 11:31'!
createWriteAccessor: aClass
	(aClass selectors includes: self writeSelector)
		ifTrue: [ ^ self ].
	aClass
		compile: (String streamContents: [ :stream |
			stream nextPutAll: self writeSelector; space; nextPutAll: 'anObject'; cr.
			stream tab; nextPutAll: self readSelector; nextPutAll: ' := anObject' ])
		classified: self categoryName! !

!MAAutoSelectorAccessor methodsFor: 'model' stamp: 'lr 3/9/2006 11:31'!
read: aModel
	(self canRead: aModel) ifFalse: [
		self createVariable: aModel class.
		self createReadAccessor: aModel class ].
	^ super read: aModel! !

!MAAutoSelectorAccessor methodsFor: 'model' stamp: 'lr 3/9/2006 11:31'!
write: anObject to: aModel
	(self canWrite: aModel) ifFalse: [
		self createVariable: aModel class.
		self createWriteAccessor: aModel class ].
	super write: anObject to: aModel! !

!MASelectorAccessor class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MASelectorAccessor class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:33'!
read: aSelector
	^ self read: aSelector write: nil! !

!MASelectorAccessor class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:33'!
read: aReadSelector write: aWriteSelector
	^ self new
		readSelector: aReadSelector;
		writeSelector: aWriteSelector;
		yourself! !

!MASelectorAccessor class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:33'!
selector: aSelector
	^ self new
		selector: aSelector;
		yourself! !

!MASelectorAccessor class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:33'!
write: aSelector
	^ self read: nil write: aSelector! !

!MASelectorAccessor methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
canRead: aModel
	^ (super canRead: aModel)
		and: [ self readSelector notNil ]
		and: [ aModel respondsTo: self readSelector ]! !

!MASelectorAccessor methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
canWrite: aModel
	^ (super canWrite: aModel)
		and: [ self writeSelector notNil ]
		and: [ aModel respondsTo: self writeSelector ]! !

!MASelectorAccessor methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:32'!
hash
	^  super hash bitXor: (self readSelector hash bitXor: self writeSelector hash)! !

!MASelectorAccessor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
readSelector
	^ readSelector! !

!MASelectorAccessor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
readSelector: aSelector
	readSelector := aSelector! !

!MASelectorAccessor methodsFor: 'model' stamp: 'lr 3/9/2006 11:32'!
read: aModel
	^ aModel perform: self readSelector! !

!MASelectorAccessor methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:32'!
selector
	^ self readSelector! !

!MASelectorAccessor methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:32'!
selector: aSelector
	self readSelector: aSelector asSymbol.
	self writeSelector: (aSelector copyWith: $:) asSymbol! !

!MASelectorAccessor methodsFor: 'printing' stamp: 'lr 3/21/2006 18:43'!
storeOn: aStream
	aStream nextPut: $(; nextPutAll: self class name; nextPutAll: ' read: '; store: self readSelector; nextPutAll: ' write: '; store: self writeSelector; nextPut: $)! !

!MASelectorAccessor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
writeSelector
	^ writeSelector! !

!MASelectorAccessor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
writeSelector: aSelector
	writeSelector := aSelector! !

!MASelectorAccessor methodsFor: 'model' stamp: 'lr 3/9/2006 11:32'!
write: anObject to: aModel
	aModel perform: self writeSelector with: anObject! !

!MASelectorAccessor methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:32'!
= anObject
	^ super = anObject
		and: [ self readSelector = anObject readSelector ]
		and: [ self writeSelector = anObject writeSelector ]! !

MAAccessor subclass: #MAVariableAccessor
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Accessor'!

!MAVariableAccessor commentStamp: 'lr 2/26/2006 10:10' prior: 0!
I am an access strategy that directly reads from and writes to instance-variables. I am strongly violating encapsulation and most of the time I should be replaced by an instance of <code>*MASelectorAccessor*</code>.!

!MAVariableAccessor class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MAVariableAccessor class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:33'!
name: aString
	^ self new
		name: aString;
		yourself! !

!MAVariableAccessor methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
canRead: aModel
	^ (super canRead: aModel)
		and: [ aModel class allInstVarNames includes: self name ]! !

!MAVariableAccessor methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
canWrite: aModel
	^ (super canWrite: aModel)
		and: [ self canRead: aModel ]! !

!MAVariableAccessor methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:32'!
hash
	^ super hash bitXor: self name hash! !

!MAVariableAccessor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
name
	^ name! !

!MAVariableAccessor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
name: aString
	name := aString! !

!MAVariableAccessor methodsFor: 'model' stamp: 'lr 3/9/2006 11:32'!
read: aModel
	^ aModel instVarNamed: self name! !

!MAVariableAccessor methodsFor: 'printing' stamp: 'lr 3/9/2006 11:32'!
storeOn: aStream
	aStream nextPut: $(; nextPutAll: self class name; nextPutAll: ' name: '; store: self name; nextPut: $)! !

!MAVariableAccessor methodsFor: 'model' stamp: 'lr 3/9/2006 11:32'!
write: anObject to: aModel
	aModel instVarNamed: self name put: anObject! !

!MAVariableAccessor methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:32'!
= anObject
	^ super = anObject
		and: [ self name = anObject name ]! !

MAObject subclass: #MACondition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Condition'!

!MACondition commentStamp: '<historical>' prior: 0!
I represent an immutable relation. Users usually want to check if the relation I am representing is satisfied by sending the message #isSatisfiedBy:!

MACondition subclass: #MAComparator
	instanceVariableNames: 'selector values'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Condition'!

!MAComparator class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isAbstract
	^ false! !

!MAComparator class methodsFor: 'instance-creation' stamp: 'lr 3/9/2006 11:32'!
selector: aSelector values: aCollection
	^ self new
		setSelector: aSelector;
		setValues: aCollection;
		yourself! !

!MAComparator methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:31'!
hash
	^ super hash bitXor: self selector hash! !

!MAComparator methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
selector
	^ selector! !

!MAComparator methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:31'!
setSelector: aSelector
	selector := aSelector! !

!MAComparator methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:31'!
setValues: aCollection
	values := aCollection! !

!MAComparator methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
values
	^ values! !

!MAComparator methodsFor: 'evaluating' stamp: 'lr 3/9/2006 11:31'!
value: anObject
	| receiverAndArguments |
	receiverAndArguments := self values collect: [ :each |
		each isAccessor
			ifFalse: [ each ]
			ifTrue: [
				(each canRead: anObject)
					ifFalse: [ ^ false ]
					ifTrue: [ each read: anObject ] ] ].
	^ receiverAndArguments first
		perform: self selector
		withArguments: receiverAndArguments allButFirst! !

!MAComparator methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:31'!
= anObject
	^ super = anObject
		and: [ self selector = anObject selector ]
		and: [ self values = anObject values ]! !

MACondition subclass: #MAComposedCondition
	instanceVariableNames: 'conditions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Condition'!

MAComposedCondition subclass: #MAAllCondition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Condition'!

!MAAllCondition class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isAbstract
	^ false! !

!MAAllCondition methodsFor: 'evaluating' stamp: 'lr 3/9/2006 11:31'!
value: anObject
	^ self conditions allSatisfy: [ :each | each value: anObject ]! !

!MAAllCondition methodsFor: 'operators' stamp: 'lr 3/9/2006 11:31'!
& aCondition
	^ self copyWith: aCondition! !

MAComposedCondition subclass: #MAAnyCondition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Condition'!

!MAAnyCondition class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isAbstract
	^ false! !

!MAAnyCondition methodsFor: 'evaluating' stamp: 'lr 3/9/2006 11:31'!
value: anObject
	^ self conditions anySatisfy: [ :each | each value: anObject ]! !

!MAAnyCondition methodsFor: 'operators' stamp: 'lr 3/9/2006 11:31'!
| aCondition
	^ self copyWith: aCondition! !

!MAComposedCondition class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:32'!
new
	^ super new
		setConditions: Array new;
		yourself! !

!MAComposedCondition class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:32'!
withAll: anArray
	^ self new
		setConditions: anArray;
		yourself! !

!MAComposedCondition class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:32'!
with: aRelation
	^ self withAll: (Array with: aRelation)! !

!MAComposedCondition class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:32'!
with: aFirstRelation with: aSecondRelation
	^ self withAll: (Array with: aFirstRelation with: aSecondRelation)! !

!MAComposedCondition methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
conditions
	^ conditions! !

!MAComposedCondition methodsFor: 'copying' stamp: 'lr 3/9/2006 11:31'!
copyWith: aRelation
	^ self class withAll: (self conditions copyWith: aRelation)! !

!MAComposedCondition methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:31'!
hash
	^ super hash bitXor: self conditions hash! !

!MAComposedCondition methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isComposed
	^ true! !

!MAComposedCondition methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:31'!
setConditions: aCollection
	conditions := aCollection! !

!MAComposedCondition methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:31'!
= anObject
	^ super = anObject
		and: [ self conditions = anObject conditions ]! !

MAComposedCondition subclass: #MANoneCondition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Condition'!

!MANoneCondition class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MANoneCondition methodsFor: 'evaluating' stamp: 'lr 3/9/2006 11:32'!
value: anObject
	^ self conditions noneSatisfy: [ :each | each value: anObject ]! !

!MANoneCondition methodsFor: 'operators' stamp: 'lr 3/9/2006 11:32'!
!! aCondition
	^ self copyWith: aCondition! !

!MACondition methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isComposed
	^ false! !

!MACondition methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isCondition
	^ true! !

!MACondition methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isConstant
	^ false! !

!MACondition methodsFor: 'operators-unary' stamp: 'lr 3/9/2006 11:32'!
not
	^ MANoneCondition with: self! !

!MACondition methodsFor: 'evaluation' stamp: 'lr 3/9/2006 11:32'!
numArgs
	"Answer the number of arguments that must be used to evaluate this condition."

	^ 1! !

!MACondition methodsFor: 'evaluation' stamp: 'lr 3/9/2006 11:32'!
value: anObject
	"Evaluates the receiver with the argument anObject."

	self subclassResponsibility! !

!MACondition methodsFor: 'operators-binary' stamp: 'lr 3/9/2006 11:32'!
!! aCondition
	^ MANoneCondition with: self with: aCondition! !

!MACondition methodsFor: 'operators-binary' stamp: 'lr 3/9/2006 11:32'!
& aCondition
	^ MAAllCondition with: self with: aCondition! !

!MACondition methodsFor: 'operators-binary' stamp: 'lr 3/9/2006 11:32'!
| aCondition
	^ MAAnyCondition with: self with: aCondition! !

MACondition subclass: #MAConstantCondition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Condition'!

!MAConstantCondition methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isConstant
	^ true! !

MAConstantCondition subclass: #MAFalseCondition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Condition'!

!MAFalseCondition class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MAFalseCondition methodsFor: 'operators' stamp: 'lr 3/9/2006 11:32'!
not
	^ MATrueCondition new! !

!MAFalseCondition methodsFor: 'evaluating' stamp: 'lr 3/9/2006 11:32'!
value: anObject
	^ false! !

!MAFalseCondition methodsFor: 'operators' stamp: 'lr 3/9/2006 11:32'!
& aCondition
	^ self! !

!MAFalseCondition methodsFor: 'operators' stamp: 'lr 3/9/2006 11:32'!
| aCondition
	^ aCondition! !

MAConstantCondition subclass: #MATrueCondition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Condition'!

!MATrueCondition class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MATrueCondition methodsFor: 'operators' stamp: 'lr 3/9/2006 11:32'!
not
	^ MAFalseCondition new! !

!MATrueCondition methodsFor: 'evaluating' stamp: 'lr 3/9/2006 11:32'!
value: anObject
	^ true! !

!MATrueCondition methodsFor: 'operators' stamp: 'lr 3/9/2006 11:32'!
& aCondition
	^ aCondition! !

!MATrueCondition methodsFor: 'operators' stamp: 'lr 3/9/2006 11:32'!
| aCondition
	^ self! !

MACondition subclass: #MASelectorCondition
	instanceVariableNames: 'selector arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Condition'!

!MASelectorCondition class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MASelectorCondition class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:33'!
selector: aSelector arguments: aCollection
	^ self new
		setSelector: aSelector;
		setArguments: aCollection;
		yourself! !

!MASelectorCondition methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
arguments
	^ arguments! !

!MASelectorCondition methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
selector
	^ selector! !

!MASelectorCondition methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:32'!
setArguments: aCollection
	arguments := aCollection! !

!MASelectorCondition methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:32'!
setSelector: aSelector
	selector := aSelector! !

!MASelectorCondition methodsFor: 'evaluating' stamp: 'lr 3/9/2006 11:32'!
value: anObject
	^ anObject perform: self selector withArguments: self arguments! !

MAObject subclass: #MADescription
	instanceVariableNames: 'accessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MADescription commentStamp: 'lr 2/26/2006 09:32' prior: 0!
I am the root of the description hierarchy in Magritte and I provide most of the basic properties avialable to all descriptions. If you would like to annotate your model with a description have a look at the different subclasses of myself.

!!Example
If your model has got an instance-variable called title, that should be used to store the title of the model-object, you could add the following description to your class:

=MyModel class>>descriptionTitle
=	^ (MAStringDescription auto: #title label: 'Title')
=		beRequired;
=		yourself.

The selector #title is the name of the accessor method used by Magritte to retrieve the value from the model. In the above case Magritte creates the accessor method and the instance-variable automatically, if necessary. The label is used to give the field a name and will be printed next to the input box if a visual GUI is created from this description.

The write-accessor is automatically deduced by adding a colon to the read-selector, in this example <code>#title:</code>. You can specify you own accessor strategy using one of the subclasses of *MAAccessor*. If you have multiple description within the same object, the priority field is used to order them. Put a low priority for description printed first.

You can then configure your description even further, some basic properties that you might want to have a look at:
- Required: Your user has to fill this field before being able to go on. Set or unset this propertie with beRequired or beOptional. Default: false.
- Readonly: It is just an information, nothing to modify here. beReadonly/beWriteable. Default: false.
- Visible: Does this description need to be displayed to the user ? beVisible/beHidden. Default: true.!

MADescription subclass: #MAContainer
	instanceVariableNames: 'children'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MAContainer commentStamp: 'lr 2/12/2006 19:34' prior: 0!
I am a container holding a collection of descriptions, all instances of subclasses of <code>*MAElementDescription*</code>. I keep my children within an <code>OrderedCollection</code>, but I don't sort them according to their priority.

I fully support the collection protocol: descriptions can be added and removed. Moreover I implement most enumeration methods, so that users are able to iterate (<code>do:</code>), filter (<code>select:</code>, <code>reject:</code>), transform (<code>collect:</code>), extract (<code>detect:</code>, <code>detect:ifNone:</code>), and test (<code>allSatisfy:</code>, <code>anySatisfy:</code>, <code>noneSatisfy:</code>) my elements.!

!MAContainer class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:32'!
defaultAccessor
	^ MAContainerAccessor new! !

!MAContainer class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:32'!
defaultCollection
	^ OrderedCollection new! !

!MAContainer class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:32'!
descriptionChildren
	^ (MAToManyRelationDescription accessor: (MASelectorAccessor read: #children write: #setChildren:) label: 'Elements' priority: 400 default: self defaultCollection)
		classes: (MADynamicObject on: [ MAElementDescription withAllConcreteClasses ]);
		yourself! !

!MAContainer class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:32'!
descriptionDefault
	^ nil! !

!MAContainer class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isAbstract
	^ false! !

!MAContainer class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:32'!
withAll: aCollection
	^ self new
		addAll: aCollection;
		yourself! !

!MAContainer class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:32'!
with: aDescription
	^ self new
		add: aDescription;
		yourself! !

!MAContainer methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitContainer: self! !

!MAContainer methodsFor: 'adding' stamp: 'lr 3/9/2006 11:32'!
addAll: aCollection
	self children addAll: aCollection! !

!MAContainer methodsFor: 'adding' stamp: 'lr 3/9/2006 11:32'!
add: aDescription
	self children add: aDescription! !

!MAContainer methodsFor: 'enumerating' stamp: 'lr 3/9/2006 11:32'!
allSatisfy: aBlock
	"Evaluate aBlock with the descriptions of the receiver. If aBlock returns false for any description return false. Otherwise return true."

	^ self children allSatisfy: aBlock! !

!MAContainer methodsFor: 'enumerating' stamp: 'lr 3/9/2006 11:32'!
anySatisfy: aBlock
	"Evaluate aBlock with the descriptions of the receiver. If aBlock returns true for any description return true. Otherwise return false."

	^ self children anySatisfy: aBlock! !

!MAContainer methodsFor: 'converting' stamp: 'lr 3/9/2006 11:32'!
asContainer
	^ self! !

!MAContainer methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
at: anIndex
	^ self children at: anIndex! !

!MAContainer methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
at: anIndex ifAbsent: aBlock
	^ self children at: anIndex ifAbsent: aBlock! !

!MAContainer methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
children
	^ children! !

!MAContainer methodsFor: 'enumerating' stamp: 'lr 3/9/2006 11:32'!
collect: aBlock
	^ self copy
		setChildren: (self children collect: aBlock);
		yourself! !

!MAContainer methodsFor: 'copying' stamp: 'lr 3/9/2006 11:32'!
copyEmpty
	^ self copy
		setChildren: self class defaultCollection;
		yourself! !

!MAContainer methodsFor: 'copying' stamp: 'lr 3/9/2006 11:32'!
copyFrom: aStartIndex to: anEndIndex
	^ self copy
		setChildren: (self children
			copyFrom: aStartIndex
			to: anEndIndex);
		yourself! !

!MAContainer methodsFor: 'enumerating' stamp: 'lr 3/9/2006 11:32'!
detect: aBlock
	^ self children detect: aBlock! !

!MAContainer methodsFor: 'enumerating' stamp: 'lr 3/9/2006 11:32'!
detect: aBlock ifNone: anExceptionBlock
	^ self children detect: aBlock ifNone: anExceptionBlock! !

!MAContainer methodsFor: 'enumerating' stamp: 'lr 3/9/2006 11:32'!
do: aBlock
	self children do: aBlock! !

!MAContainer methodsFor: 'enumerating' stamp: 'lr 3/9/2006 11:32'!
do: aBlock separatedBy: aSeparatorBlock
	self children do: aBlock separatedBy: aSeparatorBlock! !

!MAContainer methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
errorNotFound: aDescription
	self error: aDescription class label , ' not found.'! !

!MAContainer methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
hasChildren
	^ self notEmpty! !

!MAContainer methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:32'!
hash
	^ super hash bitXor: self children hash! !

!MAContainer methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
includes: aDescription
	^ self children includes: aDescription! !

!MAContainer methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:32'!
initialize
	super initialize.
	self setChildren: self class defaultCollection! !

!MAContainer methodsFor: 'enumerating' stamp: 'lr 3/9/2006 11:32'!
inject: anObject into: aBlock
	^ self children inject: anObject into: aBlock! !

!MAContainer methodsFor: 'enumerating' stamp: 'lr 3/9/2006 11:32'!
intersection: aCollection
	^ self copy
		setChildren: (self children intersection: aCollection);
		yourself! !

!MAContainer methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isContainer
	^ true! !

!MAContainer methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isEmpty
	^ self children isEmpty! !

!MAContainer methodsFor: 'moving' stamp: 'lr 3/9/2006 11:32'!
moveDown: aDescription
	self children moveDown: aDescription! !

!MAContainer methodsFor: 'moving' stamp: 'lr 3/9/2006 11:32'!
moveUp: aDescription
	self children moveUp: aDescription! !

!MAContainer methodsFor: 'enumerating' stamp: 'lr 3/9/2006 11:32'!
noneSatisfy: aBlock
	"Evaluate aBlock with the descriptions of the receiver. If aBlock returns false for all descriptions return true. Otherwise return false."

	^ self children noneSatisfy: aBlock! !

!MAContainer methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
notEmpty
	^ self children notEmpty! !

!MAContainer methodsFor: 'copying' stamp: 'lr 3/9/2006 11:32'!
postCopy
	super postCopy.
	self setChildren: self children copy! !

!MAContainer methodsFor: 'enumerating' stamp: 'lr 3/9/2006 11:32'!
reject: aBlock
	^ self copy
		setChildren: (self children reject: aBlock);
		yourself! !

!MAContainer methodsFor: 'removing' stamp: 'lr 3/9/2006 11:32'!
removeAll
	self setChildren: self class defaultCollection! !

!MAContainer methodsFor: 'removing' stamp: 'lr 3/9/2006 11:32'!
remove: aDescription
	self children remove: aDescription ifAbsent: [ self errorNotFound: aDescription ]! !

!MAContainer methodsFor: 'enumerating' stamp: 'lr 3/9/2006 11:32'!
select: aBlock
	^ self copy
		setChildren: (self children select: aBlock);
		yourself! !

!MAContainer methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:32'!
setChildren: aCollection
	children := aCollection! !

!MAContainer methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
size
	^ self children size! !

!MAContainer methodsFor: 'enumerating' stamp: 'lr 3/9/2006 11:32'!
union: aContainer
	"Answer the union of the receiver and aContainer, does not include duplicates."

	^ self copy
		addAll: (aContainer reject: [ :each | self includes: each ]);
		yourself! !

!MAContainer methodsFor: 'validating' stamp: 'lr 3/9/2006 11:32'!
validateSpecific: anObject
	| errors |
	super validateSpecific: anObject.
	errors := Array streamContents: [ :stream |
		self do: [ :each |
			each isVisible ifTrue: [
				[ each validate: (anObject readUsing: each) ]
					on: MAValidationError
					do: [ :err | stream nextPut: err ] ] ] ].
	errors isEmpty
		ifFalse: [ MAMultipleErrors on: self errors: errors ]! !

!MAContainer methodsFor: 'enumerating' stamp: 'lr 3/9/2006 11:32'!
withIndexDo: aBlock
	self children withIndexDo: aBlock! !

!MAContainer methodsFor: 'enumerating' stamp: 'lr 3/9/2006 11:32'!
with: aCollection do: aBlock
	self children with: aCollection do: aBlock! !

!MAContainer methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:32'!
= anObject
	^ super = anObject
		and: [ self children = anObject children ]! !

MAContainer subclass: #MAPriorityContainer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MAPriorityContainer commentStamp: 'lr 2/12/2006 19:34' prior: 0!
I am a container holding a collection of descriptions and I keep them sorted according to their priority.!

!MAPriorityContainer class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:33'!
defaultCollection
	^ SortedCollection new! !

!MAPriorityContainer methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitPriorityContainer: self! !

!MAPriorityContainer methodsFor: 'moving' stamp: 'lr 3/9/2006 11:32'!
moveDown: aDescription
	self shouldNotImplement! !

!MAPriorityContainer methodsFor: 'moving' stamp: 'lr 3/9/2006 11:32'!
moveUp: aDescription
	self shouldNotImplement! !

!MAPriorityContainer methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
resort
	self setChildren: self children copy! !

!MAPriorityContainer methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:32'!
setChildren: aCollection
	super setChildren: aCollection asSortedCollection! !

!MADescription class methodsFor: 'instance-creation-accessor' stamp: 'lr 3/9/2006 11:32'!
accessor: anAccessor
	^ self accessor: anAccessor label: self defaultLabel! !

!MADescription class methodsFor: 'instance-creation-accessor' stamp: 'lr 3/9/2006 11:32'!
accessor: anAccessor label: aString
	^ self accessor: anAccessor label: aString priority: self defaultPriority! !

!MADescription class methodsFor: 'instance-creation-accessor' stamp: 'lr 3/9/2006 11:32'!
accessor: anAccessor label: aString priority: aNumber
	^ self accessor: anAccessor label: aString priority: aNumber default: self defaultDefault! !

!MADescription class methodsFor: 'instance-creation-accessor' stamp: 'lr 3/9/2006 11:32'!
accessor: anAccessor label: aString priority: aNumber default: anObject
	^ self new
		accessor: anAccessor;
		label: aString;
		priority: aNumber;
		default: anObject;
		yourself! !

!MADescription class methodsFor: 'instance-creation-auto' stamp: 'lr 3/9/2006 11:32'!
auto: aSelector
	^ self accessor: (MAAutoSelectorAccessor selector: aSelector)! !

!MADescription class methodsFor: 'instance-creation-auto' stamp: 'lr 3/9/2006 11:32'!
auto: aSelector label: aString
	^ self accessor: (MAAutoSelectorAccessor selector: aSelector) label: aString! !

!MADescription class methodsFor: 'instance-creation-auto' stamp: 'lr 3/9/2006 11:32'!
auto: aSelector label: aString priority: aNumber
	^ self accessor: (MAAutoSelectorAccessor selector: aSelector) label: aString priority: aNumber! !

!MADescription class methodsFor: 'instance-creation-auto' stamp: 'lr 3/9/2006 11:32'!
auto: aSelector label: aString priority: aNumber default: anObject
	^ self accessor: (MAAutoSelectorAccessor selector: aSelector) label: aString priority: aNumber default: anObject! !

!MADescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:32'!
defaultAccessor
	^ MANullAccessor new! !

!MADescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:32'!
defaultComment
	^ nil! !

!MADescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:32'!
defaultConditions
	^ Array new! !

!MADescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:32'!
defaultDefault
	^ nil! !

!MADescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:32'!
defaultLabel
	^ String new! !

!MADescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:32'!
defaultPersistent
	^ true! !

!MADescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:32'!
defaultPriority
	^ 0! !

!MADescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:32'!
defaultReadonly
	^ false! !

!MADescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:32'!
defaultRequired
	^ false! !

!MADescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:32'!
defaultStringReader
	^ MAStringReader! !

!MADescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:32'!
defaultStringWriter
	^ MAStringWriter! !

!MADescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:32'!
defaultUndefined
	^ String new! !

!MADescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:32'!
defaultVisible
	^ true! !

!MADescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:32'!
descriptionComment
	^ MAMemoDescription selector: #comment label: 'Comment' priority: 110 default: self defaultComment! !

!MADescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:32'!
descriptionDefault
	^ self isAbstract
		ifFalse: [ self selector: #default label: 'Default' priority: 130 default: self defaultDefault ]! !

!MADescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:32'!
descriptionLabel
	^ MAStringDescription selector: #label label: 'Label' priority: 100 default: self defaultLabel! !

!MADescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:32'!
descriptionPersistent
	^ MABooleanDescription selector: #persistent label: 'Persistent' priority: 230 default: self defaultPersistent! !

!MADescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:32'!
descriptionPriority
	^ (MANumberDescription selector: #priority label: 'Priority' priority: 130 default: self defaultPriority)
		beRequired;
		yourself! !

!MADescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:32'!
descriptionReadonly
	^ MABooleanDescription selector: #readonly label: 'Readonly' priority: 200 default: self defaultReadonly! !

!MADescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:32'!
descriptionRequired
	^ MABooleanDescription selector: #required label: 'Required' priority: 220 default: self defaultRequired! !

!MADescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:32'!
descriptionStringReader
	^ (MASingleOptionDescription selector: #stringReader label: 'String Reader' priority: 300 default: self defaultStringReader)
		reference: MAClassDescription new;
		options: (MADynamicObject on: [ self defaultStringReader withAllSubclasses ]);
		yourself! !

!MADescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:32'!
descriptionStringWriter
	^ (MASingleOptionDescription selector: #stringWriter label: 'String Writer' priority: 310 default: self defaultStringWriter)
		reference: MAClassDescription new;
		options: (MADynamicObject on: [ self defaultStringWriter withAllSubclasses ]);
		yourself! !

!MADescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:32'!
descriptionUndefined
	^ MAStringDescription selector: #undefined label: 'Undefined String' priority: 140 default: self defaultUndefined! !

!MADescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:32'!
descriptionVisible
	^ MABooleanDescription selector: #visible label: 'Visible' priority: 210 default: self defaultVisible! !

!MADescription class methodsFor: 'instance-creation-null' stamp: 'lr 3/9/2006 11:32'!
null
	^ self accessor: (MANullAccessor uuid: MACompatibility uuid)! !

!MADescription class methodsFor: 'instance-creation-null' stamp: 'lr 3/9/2006 11:32'!
null: anUuid
	^ self accessor: (MANullAccessor uuid: anUuid)! !

!MADescription class methodsFor: 'instance-creation-null' stamp: 'lr 3/9/2006 11:32'!
null: anUuid label: aString
	^ self accessor: (MANullAccessor uuid: anUuid) label: aString! !

!MADescription class methodsFor: 'instance-creation-null' stamp: 'lr 3/9/2006 11:32'!
null: anUuid label: aString priority: aNumber
	^ self accessor: (MANullAccessor uuid: anUuid) label: aString priority: aNumber! !

!MADescription class methodsFor: 'instance-creation-null' stamp: 'lr 3/9/2006 11:32'!
null: anUuid label: aString priority: aNumber default: anObject
	^ self accessor: (MANullAccessor uuid: anUuid) label: aString priority: aNumber default: anObject! !

!MADescription class methodsFor: 'instance-creation-block' stamp: 'lr 3/9/2006 11:32'!
read: aReadBlock write: aWriteBlock
	^ self accessor: (MABlockAccessor read: aReadBlock write: aWriteBlock)! !

!MADescription class methodsFor: 'instance-creation-block' stamp: 'lr 3/9/2006 11:32'!
read: aReadBlock write: aWriteBlock label: aString
	^ self accessor: (MABlockAccessor read: aReadBlock write: aWriteBlock) label: aString! !

!MADescription class methodsFor: 'instance-creation-block' stamp: 'lr 3/9/2006 11:32'!
read: aReadBlock write: aWriteBlock label: aString priority: aNumber
	^ self accessor: (MABlockAccessor read: aReadBlock write: aWriteBlock) label: aString priority: aNumber! !

!MADescription class methodsFor: 'instance-creation-block' stamp: 'lr 3/9/2006 11:32'!
read: aReadBlock write: aWriteBlock label: aString priority: aNumber default: anObject
	^ self accessor: (MABlockAccessor read: aReadBlock write: aWriteBlock) label: aString priority: aNumber default: anObject! !

!MADescription class methodsFor: 'instance-creation-cascade' stamp: 'lr 3/9/2006 11:32'!
selectors: anArray
	^ self accessor: (MACascadeAccessor selectors: anArray)! !

!MADescription class methodsFor: 'instance-creation-cascade' stamp: 'lr 3/9/2006 11:32'!
selectors: anArray label: aString
	^ self accessor: (MACascadeAccessor selectors: anArray) label: aString! !

!MADescription class methodsFor: 'instance-creation-cascade' stamp: 'lr 3/9/2006 11:32'!
selectors: anArray label: aString priority: aNumber
	^ self accessor: (MACascadeAccessor selectors: anArray) label: aString priority: aNumber! !

!MADescription class methodsFor: 'instance-creation-cascade' stamp: 'lr 3/9/2006 11:32'!
selectors: anArray label: aString priority: aNumber default: anObject
	^ self accessor: (MACascadeAccessor selectors: anArray) label: aString priority: aNumber default: anObject! !

!MADescription class methodsFor: 'instance-creation-selector' stamp: 'lr 3/9/2006 11:32'!
selector: aSelector
	^ self accessor: (MASelectorAccessor selector: aSelector)! !

!MADescription class methodsFor: 'instance-creation-selector' stamp: 'lr 3/9/2006 11:32'!
selector: aSelector label: aString
	^ self accessor: (MASelectorAccessor selector: aSelector) label: aString! !

!MADescription class methodsFor: 'instance-creation-selector' stamp: 'lr 3/9/2006 11:32'!
selector: aSelector label: aString priority: aNumber
	^ self accessor: (MASelectorAccessor selector: aSelector) label: aString priority: aNumber! !

!MADescription class methodsFor: 'instance-creation-selector' stamp: 'lr 3/9/2006 11:32'!
selector: aSelector label: aString priority: aNumber default: anObject
	^ self accessor: (MASelectorAccessor selector: aSelector) label: aString priority: aNumber default: anObject! !

!MADescription class methodsFor: 'instance-creation-variable' stamp: 'lr 3/9/2006 11:32'!
variable: aSymbol
	^ self accessor: (MAVariableAccessor name: aSymbol)! !

!MADescription class methodsFor: 'instance-creation-variable' stamp: 'lr 3/9/2006 11:32'!
variable: aSymbol label: aString
	^ self accessor: (MAVariableAccessor name: aSymbol) label: aString! !

!MADescription class methodsFor: 'instance-creation-variable' stamp: 'lr 3/9/2006 11:32'!
variable: aSymbol label: aString priority: aNumber
	^ self accessor: (MAVariableAccessor name: aSymbol) label: aString priority: aNumber! !

!MADescription class methodsFor: 'instance-creation-variable' stamp: 'lr 3/9/2006 11:32'!
variable: aSymbol label: aString priority: aNumber default: anObject
	^ self accessor: (MAVariableAccessor name: aSymbol) label: aString priority: aNumber default: anObject! !

!MADescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
version
	^ 1! !

!MADescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitDescription: self! !

!MADescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
accessor
	"Answer the access-strategy of the model-value described by the receiver."

	^ accessor ifNil: [ accessor := self class defaultAccessor ]! !

!MADescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
accessor: anAccessor
	accessor := anAccessor! !

!MADescription methodsFor: 'validation' stamp: 'lr 3/9/2006 11:32'!
addCondition: aCondition
	self addCondition: aCondition labelled: aCondition asString! !

!MADescription methodsFor: 'validation' stamp: 'lr 3/9/2006 11:32'!
addCondition: aCondition labelled: aString
	"Add aCondition as an additional validation condition to the receiver and give it the label aString. The first argument is either a block-context, a composite of the subclasses of *MACondition*, or any other object that responds to #value: with true or false."

	self conditions: (self conditions
		copyWith: (Association
			key: aCondition
			value: aString))! !

!MADescription methodsFor: 'converting' stamp: 'lr 3/9/2006 11:32'!
asContainer
	"Answer a description container of the receiver."

	self subclassResponsibility! !

!MADescription methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
beEphemeral
	self persistent: false! !

!MADescription methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
beHidden
	self visible: false! !

!MADescription methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
beOptional
	self required: false! !

!MADescription methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
bePersistent
	self persistent: true! !

!MADescription methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
beReadonly
	self readonly: true! !

!MADescription methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
beRequired
	self required: true! !

!MADescription methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
beVisible
	self visible: true! !

!MADescription methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
beWriteable
	self readonly: false! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
comment
	"Answer a comment or help-text giving a hint what this description is used for. GUIs that are built from this description might display it as a tool-tip."

	^ self propertyAt: #comment ifAbsent: [ self class defaultComment ]! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
comment: aString
	self propertyAt: #comment put: aString! !

!MADescription methodsFor: 'accessing-configuration' stamp: 'lr 3/9/2006 11:32'!
comparators
	"Answer a collection of comparators that are usable with the described object."

	^ MATrueCondition new! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
conditions
	"Answer a collection of additional conditions that need to be fulfilled so that the described model is valid. Internally the collection associates conditions, that are either blocks or subclasses of *MACondition*, with an error string."

	^ self propertyAt: #conditions ifAbsent: [ self class defaultConditions ]! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
conditions: anArray
	self propertyAt: #conditions put: anArray! !

!MADescription methodsFor: 'strings' stamp: 'lr 3/9/2006 11:32'!
fromStringCollection: aCollection
	"Answer a collection of objects being parsed from aCollection of strings."

	^ self fromStringCollection: aCollection reader: self stringReader! !

!MADescription methodsFor: 'strings' stamp: 'lr 3/9/2006 11:32'!
fromStringCollection: aCollection reader: aParser
	"Answer a collection of objects being parsed from aCollection of strings using aParser."

	^ aCollection collect: [ :each | self fromString: each reader: aParser ]! !

!MADescription methodsFor: 'strings' stamp: 'lr 3/9/2006 11:32'!
fromString: aString
	"Answer an object being parsed from aString."

	^ self fromString: aString reader: self stringReader! !

!MADescription methodsFor: 'strings' stamp: 'lr 3/9/2006 11:32'!
fromString: aString reader: aParser
	"Answer an object being parsed from aString using aParser."

	^ aParser read: aString readStream description: self! !

!MADescription methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
hasChildren
	"Answer true if the the reciever has got any child-descriptions, e.g. a description container usually has children."

	^ false! !

!MADescription methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
hasComment
	"Answer true if the the reciever has got a non empty comment."

	^ self comment isEmptyOrNil not! !

!MADescription methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:32'!
hash
	^ super hash bitXor: self accessor hash! !

!MADescription methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
hasLabel
	"Answer true if the the reciever has got a non empty label."

	^ self label isEmptyOrNil not! !

!MADescription methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isContainer
	"Answer true if the reciever is a description container."

	^ false! !

!MADescription methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isDescription
	"Answer true if the reciever is a description."

	^ true! !

!MADescription methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isPersistent
	"Answer true if the described model is persistent."

	^ self persistent! !

!MADescription methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isReadonly
	"Answer true if the model described by the reciever is readonly."

	^ self readonly! !

!MADescription methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isRequired
	"Answer true if the model described by the reciever is required, e.g. it cannot be nil."

	^ self required! !

!MADescription methodsFor: 'validation' stamp: 'lr 3/9/2006 11:32'!
isSatisfiedBy: anObject
	"Answer true if anObject is a valid instance of the receivers description."

	[ self validate: anObject ]
		on: MAValidationError
		do: [ :err | ^ false ].
	^ true! !

!MADescription methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isVisible
	"Answer true if the model described by the reciever is visible, as an opposite to hidden."

	^ self visible! !

!MADescription methodsFor: 'accessing-configuration' stamp: 'lr 3/9/2006 11:32'!
kind
	"Answer the base-class (type) the receiver is describing. The default implementation answers the most generic class: Object, the root of the Smalltalk class hierarchy. Subclasses might refine this choice."

	^ Object! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
label
	"Answer the label of the receiving description. The label is mostly used as an identifier that is printed next to the input field when building a GUI from the receiver."

	^ self propertyAt: #label ifAbsent: [ self class defaultLabel ]! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
label: aString
	self propertyAt: #label put: aString! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
persistent
	"Answer true if the model described by the reciever is persistent."

	^ self propertyAt: #persistent ifAbsent: [ self class defaultPersistent ]! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
persistent: aBoolean
	self propertyAt: #persistent put: aBoolean! !

!MADescription methodsFor: 'copying' stamp: 'lr 3/9/2006 11:32'!
postCopy
	super postCopy.
	self accessor: self accessor copy! !

!MADescription methodsFor: 'printing' stamp: 'lr 3/9/2006 11:32'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' label: '; print: self label.
	aStream nextPutAll: ' comment: '; print: self comment! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
priority
	"Answer a number that is the priority of the receiving description. Priorites are used to give descriptions an explicit order by sorting them according to their priorites."

	^ self propertyAt: #priority ifAbsent: [ self class defaultPriority ]! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
priority: aNumber
	self propertyAt: #priority put: aNumber! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
readonly
	"Answer true if the model described by the reciever is readonly."

	^ self propertyAt: #readonly ifAbsent: [ self class defaultReadonly ]! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
readonly: aBoolean
	self propertyAt: #readonly put: aBoolean! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
required
	"Answer true if the model described by the reciever is required, e.g. it cannot be nil."

	^ self propertyAt: #required ifAbsent: [ self class defaultRequired ]! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
required: aBoolean
	self propertyAt: #required put: aBoolean! !

!MADescription methodsFor: 'accessing-strings' stamp: 'lr 3/9/2006 11:32'!
stringReader
	"Answer a vistor-strategy that is used to parse the model described by the receiver from a string."

	^ self propertyAt: #stringReader ifAbsent: [ self class defaultStringReader ]! !

!MADescription methodsFor: 'accessing-strings' stamp: 'lr 3/9/2006 11:32'!
stringReader: aClass
	self propertyAt: #stringReader put: aClass! !

!MADescription methodsFor: 'accessing-strings' stamp: 'lr 3/9/2006 11:32'!
stringWriter
	"Answer a vistor-strategy that is used to convert the model described by the receiver to a string."

	^ self propertyAt: #stringWriter ifAbsent: [ self class defaultStringWriter ]! !

!MADescription methodsFor: 'accessing-strings' stamp: 'lr 3/9/2006 11:32'!
stringWriter: aClass
	self propertyAt: #stringWriter put: aClass! !

!MADescription methodsFor: 'strings' stamp: 'lr 3/9/2006 11:32'!
toStringCollection: aCollection
	"Answer a collection of strings being formatted from aCollection."

	^ self toStringCollection: aCollection writer: self stringWriter! !

!MADescription methodsFor: 'strings' stamp: 'lr 3/9/2006 11:32'!
toStringCollection: aCollection writer: aFormatter
	"Answer a collection of strings being formatted from aCollection using aFormatter."

	^ aCollection collect: [ :each | self toString: each writer: aFormatter ]! !

!MADescription methodsFor: 'strings' stamp: 'lr 3/9/2006 11:32'!
toString: anObject
	"Answer a string being formatted from anObject."

	^ self toString: anObject writer: self stringWriter! !

!MADescription methodsFor: 'strings' stamp: 'lr 3/9/2006 11:32'!
toString: anObject writer: aFormatter
	"Answer a string being formatted from anObject using aFormatter."

	^ aFormatter write: anObject description: self! !

!MADescription methodsFor: 'accessing-strings' stamp: 'lr 3/9/2006 11:32'!
undefined
	"Answer a string that is printed whenever the model described by the receiver is nil."

	^ (self propertyAt: #undefined ifAbsent: [ self class defaultUndefined ])
		ifNil: [ self class defaultUndefined ]! !

!MADescription methodsFor: 'accessing-strings' stamp: 'lr 3/9/2006 11:32'!
undefined: aString
	self propertyAt: #undefined put: aString! !

!MADescription methodsFor: 'validation' stamp: 'lr 3/9/2006 11:32'!
validateConditions: anObject
	"Validate anObject to satisfy all its custom conditions."

	self conditions do: [ :each |
		(each key value: anObject)
			ifFalse: [ MAConditionError on: self explanation: each value ] ]! !

!MADescription methodsFor: 'validation' stamp: 'lr 3/9/2006 11:32'!
validateKind: anObject
	"Validate anObject to be of the right kind."

	(anObject isKindOf: self kind)
		ifFalse: [ MAKindError on: self ]! !

!MADescription methodsFor: 'validation' stamp: 'lr 3/21/2006 18:43'!
validateRequired: anObject
	"Validate anObject not to be nil if it is required."

	(self isRequired and: [ anObject isNil ])
		ifTrue: [ MARequiredError on: self ]! !

!MADescription methodsFor: 'validation' stamp: 'lr 3/9/2006 11:32'!
validateSpecific: anObject
	"Validate anObject to satisfy all its description specific validation rules. Subclasses mostly want to override this method."! !

!MADescription methodsFor: 'validation' stamp: 'lr 3/9/2006 11:32'!
validate: anObject
	"Validate anObject in the context of the describing-receiver, raises an error in case of a problem. If anObject is nil and not required, most tests will be skipped. Do not override this message, instead have a look at #validateSpecific: what is usually a better place to define the behaviour your description requires."

	self validateRequired: anObject.
	anObject isNil ifFalse: [
		self
			validateKind: anObject;
			validateSpecific: anObject;
			validateConditions: anObject ]! !

!MADescription methodsFor: 'validation' stamp: 'lr 3/9/2006 11:32'!
validate: anObject description: aDescription
	[ aDescription validate: anObject ]
		on: MAValidationError
		do: [ :err | err setDescription: self; pass ]! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
visible
	"Answer true if the model described by the reciever is visible, as an opposite to hidden."

	^ self propertyAt: #visible ifAbsent: [ self class defaultVisible ]! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
visible: aBoolean
	self propertyAt: #visible put: aBoolean! !

!MADescription methodsFor: 'operators' stamp: 'lr 3/9/2006 11:32'!
, aDescription
	"Concatenate the receiver and aDescription to one composed description. Answer a description container containing both descriptions."

	^ self asContainer copy
		addAll: aDescription asContainer;
		yourself! !

!MADescription methodsFor: 'operators' stamp: 'lr 3/9/2006 11:32'!
<= anObject
	"Answer whether the receiver should precede anObject in a priority container."

	^ self priority <= anObject priority! !

!MADescription methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:32'!
= anObject
	^ super = anObject
		and: [ self accessor = anObject accessor ]! !

MADescription subclass: #MAElementDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MAElementDescription commentStamp: '<historical>' prior: 0!
I am an abstract description for all basic description types.!

MAElementDescription subclass: #MABooleanDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MABooleanDescription commentStamp: '<historical>' prior: 0!
I am a description of the boolean values <code>true</code> and <code>false</code>. My visual representation could be a check-box.!

!MABooleanDescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:32'!
defaultFalseString
	^ self defaultFalseStrings first! !

!MABooleanDescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:32'!
defaultFalseStrings
	^ #( 'false' 'f' 'no' 'n' '0' 'off' )! !

!MABooleanDescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:32'!
defaultTrueString
	^ self defaultTrueStrings first! !

!MABooleanDescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:32'!
defaultTrueStrings
	^ #( 'true' 't' 'yes' 'y' '1' 'on' )! !

!MABooleanDescription class methodsFor: 'accessing-descriptions' stamp: 'lr 3/9/2006 11:32'!
descriptionFalseString
	^ MAStringDescription selector: #falseString label: 'False String' priority: 410 default: self defaultFalseString! !

!MABooleanDescription class methodsFor: 'accessing-descriptions' stamp: 'lr 3/9/2006 11:32'!
descriptionRequired
	^ nil! !

!MABooleanDescription class methodsFor: 'accessing-descriptions' stamp: 'lr 3/9/2006 11:32'!
descriptionTrueString
	^ MAStringDescription selector: #trueString label: 'True String' priority: 400 default: self defaultTrueString! !

!MABooleanDescription class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isAbstract
	^ false! !

!MABooleanDescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
label
	^ 'Boolean'! !

!MABooleanDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:31'!
acceptMagritte: aVisitor
	aVisitor visitBooleanDescription: self! !

!MABooleanDescription methodsFor: 'accessing-selection' stamp: 'lr 3/9/2006 11:31'!
allOptions
	^ self options! !

!MABooleanDescription methodsFor: 'accessing-selection' stamp: 'lr 3/9/2006 11:31'!
allOptionsWith: anObject
	^ self options! !

!MABooleanDescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:31'!
falseString
	^ self propertyAt: #falseString ifAbsent: [ self class defaultFalseString ]! !

!MABooleanDescription methodsFor: 'accessing-readonly' stamp: 'lr 3/9/2006 11:31'!
falseStrings
	^ self class defaultFalseStrings! !

!MABooleanDescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:31'!
falseString: aString
	^ self propertyAt: #falseString put: aString! !

!MABooleanDescription methodsFor: 'accessing-selection' stamp: 'lr 3/9/2006 11:31'!
isExtensible
	^ false! !

!MABooleanDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
kind
	^ Boolean! !

!MABooleanDescription methodsFor: 'accessing-selection' stamp: 'lr 3/9/2006 11:31'!
options
	^ Array with: false with: true! !

!MABooleanDescription methodsFor: 'accessing-selection' stamp: 'lr 3/9/2006 11:31'!
reference
	^ self! !

!MABooleanDescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:31'!
trueString
	^ self propertyAt: #trueString ifAbsent: [ self class defaultTrueString ]! !

!MABooleanDescription methodsFor: 'accessing-readonly' stamp: 'lr 3/9/2006 11:31'!
trueStrings
	^ self class defaultTrueStrings! !

!MABooleanDescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:31'!
trueString: aString
	^ self propertyAt: #trueString put: aString! !

MAElementDescription subclass: #MAClassDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MAClassDescription commentStamp: '<historical>' prior: 0!
I am a description of Smalltalk classes, possible values can be any of <code>Smalltalk allClasses</code>.!

!MAClassDescription class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isAbstract
	^ false! !

!MAClassDescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
label
	^ 'Class'! !

!MAClassDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:31'!
acceptMagritte: aVisitor
	aVisitor visitClassDescription: self! !

!MAClassDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
kind
	^ Class! !

MAElementDescription subclass: #MAColorDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MAColorDescription commentStamp: '<historical>' prior: 0!
I am a description of colors, possible values are instances of <code>Color</code>. My visual representation could be a color-chooser.!

!MAColorDescription class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isAbstract
	^ false! !

!MAColorDescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
label
	^ 'Color'! !

!MAColorDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:31'!
acceptMagritte: aVisitor
	aVisitor visitColorDescription: self! !

!MAColorDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
kind
	^ Color! !

!MAElementDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitElementDescription: self! !

!MAElementDescription methodsFor: 'converting' stamp: 'lr 3/9/2006 11:32'!
asContainer
	^ MAContainer with: self! !

!MAElementDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
default
	^ self propertyAt: #default ifAbsent: [ self class defaultDefault ]! !

!MAElementDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
default: anObject
	self propertyAt: #default put: anObject! !

MAElementDescription subclass: #MAFileDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MAFileDescription commentStamp: '<historical>' prior: 0!
I am a description of files, their contents, filename and mime-type. Possible values include instances of <code>*MAFileModel*</code>. My visual representation could be a file-upload dialog.!

!MAFileDescription class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MAFileDescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
label
	^ 'File'! !

!MAFileDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitFileDescription: self! !

!MAFileDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
kind
	^ MAFileModel! !

MAElementDescription subclass: #MAMagnitudeDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MAMagnitudeDescription commentStamp: '<historical>' prior: 0!
I am an abstract description for subclasses of <code>Magnitude</code>. The range of accepted values can be limited using the accessors <code>min:</code> and <code>max:</code>.!

MAMagnitudeDescription subclass: #MADateDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MADateDescription commentStamp: '<historical>' prior: 0!
I am a description of dates, possible values are instances of <code>Date</code>. My visual representation could be a date-picker.!

!MADateDescription class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isAbstract
	^ false! !

!MADateDescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
label
	^ 'Date'! !

!MADateDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitDateDescription: self! !

!MADateDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
kind
	^ Date! !

MAMagnitudeDescription subclass: #MADurationDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MADurationDescription commentStamp: '<historical>' prior: 0!
I am a description of durations, possible values are instances of <code>Duration</code>.!

!MADurationDescription class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MADurationDescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
label
	^ 'Duration'! !

!MADurationDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitDurationDescription: self! !

!MADurationDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
kind
	^ Duration! !

!MAMagnitudeDescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:33'!
defaultMax
	^ nil! !

!MAMagnitudeDescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:33'!
defaultMin
	^ nil! !

!MAMagnitudeDescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:33'!
descriptionMax
	^ self selector: #max label: 'Maximum' priority: 410! !

!MAMagnitudeDescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:33'!
descriptionMin
	^ self selector: #min label: 'Min' priority: 400! !

!MAMagnitudeDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitMagnitudeDescription: self! !

!MAMagnitudeDescription methodsFor: 'accessing-configuration' stamp: 'lr 3/9/2006 11:32'!
comparators
	^ Array
		with: (MAComparator
			selector: #>
			descriptions: (Array
				with: (self class new label: 'greater than')))
		with: (MAComparator
			selector: #<
			descriptions: (Array
				with: (self class new label: 'less than')))
		with: (MAComparator
			selector: #between:and:
			descriptions: (Array
				with: (self class new label: 'between')
				with: (self class new label: 'and')))! !

!MAMagnitudeDescription methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isWithinRange: anObject
	^ (self min isNil or: [ self min <= anObject ])
		and: [ self max isNil or: [ self max >= anObject ] ]! !

!MAMagnitudeDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
max
	^ self propertyAt: #max ifAbsent: [ self class defaultMax ]! !

!MAMagnitudeDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
max: aMagnitudeOrNil
	"Set the maximum for accepted values, or nil if open."

	^ self propertyAt: #max put: aMagnitudeOrNil! !

!MAMagnitudeDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
min
	^ self propertyAt: #min ifAbsent: [ self class defaultMin ]! !

!MAMagnitudeDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
min: aMagnitudeOrNil
	"Set the minimum for accepted values, or nil if open."

	^ self propertyAt: #min put: aMagnitudeOrNil! !

!MAMagnitudeDescription methodsFor: 'conveniance' stamp: 'lr 3/9/2006 11:32'!
min: aMinimumObject max: aMaximumObject
	"Set the minimum and maximum of accepted values, or nil if open."

	self min: aMinimumObject; max: aMaximumObject! !

!MAMagnitudeDescription methodsFor: 'validation' stamp: 'lr 3/9/2006 11:32'!
validateSpecific: anObject
	super validateSpecific: anObject.
	(self isWithinRange: anObject)
		ifFalse: [ MARangeError on: self ]! !

MAMagnitudeDescription subclass: #MANumberDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MANumberDescription commentStamp: 'lr 2/12/2006 19:36' prior: 0!
I am a description of numbers, possible values are instances of <code>Number</code> and all its subclasses, including <code>Integer</code> and <code>Float</code>. My visual representation could be a number input-box or even a slider-control.!

!MANumberDescription class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MANumberDescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
label
	^ 'Number'! !

!MANumberDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitNumberDescription: self! !

!MANumberDescription methodsFor: 'conveniance' stamp: 'lr 3/9/2006 11:32'!
beInteger
	self addCondition: [ :value | value isInteger ] asCondition labelled: 'No integer was entered'! !

!MANumberDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
kind
	^ Number! !

MAMagnitudeDescription subclass: #MATimeDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MATimeDescription commentStamp: '<historical>' prior: 0!
I am a description of times, possible values are instances of <code>Time</code>. My visual representation could be a time-picker.!

!MATimeDescription class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MATimeDescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
label
	^ 'Time'! !

!MATimeDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitTimeDescription: self! !

!MATimeDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
kind
	^ Time! !

MAMagnitudeDescription subclass: #MATimeStampDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MATimeStampDescription commentStamp: '<historical>' prior: 0!
I am a description of timestamps, possible values are instances of <code>TimeStamp</code>. My visual representation could be a date- and time-picker.!

!MATimeStampDescription class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MATimeStampDescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
label
	^ 'Timestamp'! !

!MATimeStampDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitTimeStampDescription: self! !

!MATimeStampDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
kind
	^ TimeStamp! !

MAElementDescription subclass: #MAReferenceDescription
	instanceVariableNames: 'reference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MAReferenceDescription commentStamp: 'lr 2/12/2006 19:38' prior: 0!
I am an abstract superclass for descriptions holding onto another description.
!

MAReferenceDescription subclass: #MAOptionDescription
	instanceVariableNames: 'options'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MAOptionDescription commentStamp: '<historical>' prior: 0!
I am an abstract description of different options the user can choose from. My instance-variable <code>options</code> holds onto the options I am representing. The options can be sorted or unsorted.!

MAOptionDescription subclass: #MAMultipleOptionDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MAMultipleOptionDescription commentStamp: 'lr 2/12/2006 19:39' prior: 0!
I am a description of multiple options, possible options are stored within the <code>options</code> field, possible values are instances of <code>Collection</code>. My visual representation could be a multi-select list or a group of check-boxes.!

!MAMultipleOptionDescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:33'!
defaultDistinct
	^ false! !

!MAMultipleOptionDescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:33'!
defaultOrdered
	^ false! !

!MAMultipleOptionDescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:33'!
descriptionDistinct
	^ MABooleanDescription selector: #distinct label: 'Distinct' priority: 250 default: self defaultDistinct! !

!MAMultipleOptionDescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:33'!
descriptionOrdered
	^ MABooleanDescription selector: #ordered label: 'Ordered' priority: 260 default: self defaultOrdered! !

!MAMultipleOptionDescription class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MAMultipleOptionDescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
label
	^ 'Multiple-Option'! !

!MAMultipleOptionDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitMultipleOptionDescription: self! !

!MAMultipleOptionDescription methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
beDistinct
	self distinct: true! !

!MAMultipleOptionDescription methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
beIndefinite
	self distinct: false! !

!MAMultipleOptionDescription methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
beOrdered
	self ordered: true! !

!MAMultipleOptionDescription methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
beUnordered
	self ordered: false! !

!MAMultipleOptionDescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
distinct
	^ self propertyAt: #distinct ifAbsent: [ self class defaultDistinct ]! !

!MAMultipleOptionDescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
distinct: aBoolean
	self propertyAt: #distinct put: aBoolean! !

!MAMultipleOptionDescription methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isDistinct
	^ self distinct! !

!MAMultipleOptionDescription methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isOrdered
	^ self ordered! !

!MAMultipleOptionDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
kind
	^ Collection! !

!MAMultipleOptionDescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
ordered
	^ self propertyAt: #ordered ifAbsent: [ self class defaultOrdered ]! !

!MAMultipleOptionDescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
ordered: aBoolean
	self propertyAt: #ordered put: aBoolean! !

!MAMultipleOptionDescription methodsFor: 'validating' stamp: 'lr 3/9/2006 11:32'!
validateKind: anObject
	super validateKind: anObject.
	(anObject allSatisfy: [ :each | self options includes: each ])
		ifFalse: [ MAKindError on: self ]! !

!MAMultipleOptionDescription methodsFor: 'validating' stamp: 'lr 3/9/2006 11:32'!
validateRequired: anObject
	super validateRequired: anObject.
	(self isRequired and: [ anObject isCollection ] and: [ anObject isEmpty ])
		ifTrue: [ MARequiredError on: self ]! !

!MAMultipleOptionDescription methodsFor: 'validating' stamp: 'lr 3/9/2006 11:32'!
validateSpecific: anObject
	super validateSpecific: anObject.
	anObject do: [ :each | self validate: each description: self reference ]! !

!MAOptionDescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:33'!
defaultOptions
	^ OrderedCollection new! !

!MAOptionDescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:33'!
defaultSorted
	^ false! !

!MAOptionDescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:33'!
descriptionDefault
	^ nil! !

!MAOptionDescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:33'!
descriptionOptions
	^ MAMemoDescription selector: #optionsTextual label: 'Options' priority: 410 default: self defaultOptions! !

!MAOptionDescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:33'!
descriptionSorted
	^ MABooleanDescription selector: #sorted label: 'Sorted' priority: 240 default: self defaultSorted! !

!MAOptionDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitOptionDescription: self! !

!MAOptionDescription methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:32'!
allOptions
	^ self prepareOptions: self options copy! !

!MAOptionDescription methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:32'!
allOptionsWith: anObject
	^ self prepareOptions: ((anObject isNil
		or: [ self options includes: anObject ])
			ifTrue: [ self options copy ]
			ifFalse: [ self options copyWith: anObject ])! !

!MAOptionDescription methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
beSorted
	self sorted: true! !

!MAOptionDescription methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
beUnsorted
	self sorted: false! !

!MAOptionDescription methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isSorted
	^ self sorted! !

!MAOptionDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
options
	^ options ifNil: [ options := self class defaultOptions ]! !

!MAOptionDescription methodsFor: 'accessing-textual' stamp: 'lr 3/9/2006 11:32'!
optionsTextual
	^ (self reference toStringCollection: self options)
		asMultilineString! !

!MAOptionDescription methodsFor: 'accessing-textual' stamp: 'lr 3/9/2006 11:32'!
optionsTextual: aString
	^ self options: (self reference fromStringCollection: aString lines)! !

!MAOptionDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
options: anArray
	options := anArray! !

!MAOptionDescription methodsFor: 'copying' stamp: 'lr 3/9/2006 11:32'!
postCopy
	super postCopy.
	self options: self options copy! !

!MAOptionDescription methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
prepareOptions: aCollection
	^ aCollection asArray in: [ :result |
		self isSorted
			ifTrue: [ result sort: self sortBlock ]
			ifFalse: [ result ] ]! !

!MAOptionDescription methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
sortBlock
	^ [ :a :b | (self reference toString: a) caseInsensitiveLessOrEqual: (self reference toString: b) ]! !

!MAOptionDescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
sorted
	^ self propertyAt: #sorted ifAbsent: [ self class defaultSorted ]! !

!MAOptionDescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
sorted: aBoolean
	^ self propertyAt: #sorted put: aBoolean! !

!MAOptionDescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
undefined: aString
	super undefined: aString.
	self reference isNil
		ifFalse: [ self reference undefined: aString ]! !

MAOptionDescription subclass: #MASingleOptionDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MASingleOptionDescription commentStamp: 'lr 2/12/2006 19:40' prior: 0!
I am a description of a single option, possible values are stored within the <code>options</code> field, but I might also be extensible so that the user can add its own option. My visual representation could be a drop-down list or a group of option-buttons.!

!MASingleOptionDescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:33'!
defaultExtensible
	^ false! !

!MASingleOptionDescription class methodsFor: 'accessing-descriptions' stamp: 'lr 3/9/2006 11:33'!
descriptionExtensible
	^ MABooleanDescription selector: #extensible label: 'Extensible' priority: 250 default: self defaultExtensible! !

!MASingleOptionDescription class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MASingleOptionDescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
label
	^ 'Single-Option'! !

!MASingleOptionDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitSingleOptionDescription: self! !

!MASingleOptionDescription methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
beExtensible
	self extensible: true! !

!MASingleOptionDescription methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
beLimited
	self extensible: false! !

!MASingleOptionDescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
extensible
	^ self propertyAt: #extensible ifAbsent: [ self class defaultExtensible ]! !

!MASingleOptionDescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
extensible: aBoolean
	self propertyAt: #extensible put: aBoolean! !

!MASingleOptionDescription methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isExtensible
	^ self extensible! !

!MASingleOptionDescription methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
prepareOptions: aCollection
	^ self isRequired
		ifTrue: [ super prepareOptions: aCollection ]
		ifFalse: [
			(super prepareOptions: aCollection)
				copyWithFirst: nil ]! !

!MASingleOptionDescription methodsFor: 'validating' stamp: 'lr 3/9/2006 11:32'!
validateKind: anObject
	super validateKind: anObject.
	(self isExtensible or: [ self options includes: anObject ])
		ifFalse: [ MAKindError on: self ]! !

!MASingleOptionDescription methodsFor: 'validating' stamp: 'lr 3/9/2006 11:32'!
validateSpecific: anObject
	super validateSpecific: anObject.
	self validate: anObject description: self reference! !

!MAReferenceDescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:33'!
defaultReference
	^ MAStringDescription new! !

!MAReferenceDescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:33'!
descriptionReference
	^ (MAToOneRelationDescription selector: #reference label: 'Description' priority: 400)
		classes: (MADynamicObject on: [ MADescription withAllConcreteClasses ]);
		beRequired;
		yourself! !

!MAReferenceDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitReferenceDescription: self! !

!MAReferenceDescription methodsFor: 'copying' stamp: 'lr 3/9/2006 11:32'!
postCopy
	super postCopy.
	self reference: self reference copy! !

!MAReferenceDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
reference
	^ reference ifNil: [ reference := self class defaultReference ]! !

!MAReferenceDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
reference: aDescription
	reference := aDescription! !

MAReferenceDescription subclass: #MARelationDescription
	instanceVariableNames: 'classes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MARelationDescription commentStamp: 'lr 2/12/2006 19:42' prior: 0!
I am an abstract description for descriptions representing a relation. My instance-variable <code>classes</code> holds onto a collection of possible classes that I can relate to. If required the reference description will be automatically built from this list of classes.!

!MARelationDescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:33'!
defaultClasses
	^ Set new! !

!MARelationDescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:33'!
defaultReference
	^ nil! !

!MARelationDescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:33'!
descriptionClasses
	^ (MAMultipleOptionDescription selector: #classes label: 'Classes' priority: 400)
		options: (MADynamicObject on: [ Smalltalk allClasses ]);
		reference: MAClassDescription new;
		yourself! !

!MARelationDescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:33'!
descriptionReference
	^ super descriptionReference
		classes: (MADynamicObject on: [ MAContainer withAllConcreteClasses ]);
		beOptional;
		yourself! !

!MARelationDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitRelationDescription: self! !

!MARelationDescription methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:32'!
allClasses
	^ (Array withAll: self classes)
		sort: [ :a :b | a label caseInsensitiveLessOrEqual: b label ];
		yourself! !

!MARelationDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
classes
	^ classes ifNil: [ classes := self class defaultClasses ]! !

!MARelationDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
classes: aCollection
	classes := aCollection! !

!MARelationDescription methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:32'!
commonClass
	"Answer a common superclass of the classes of the receiver. The algorithm is implemented to be as efficient as possible, the inner loop will be only execute the first few iterations."

	| current |
	self classes isEmpty
		ifTrue: [ ^ self class defaultContainer ].
	current := self classes anyOne.
	self classes do: [ :each |
		[ each includesBehavior: current ]
			whileFalse: [ current := current superclass ] ].
	^ current! !

!MARelationDescription methodsFor: 'copying' stamp: 'lr 3/9/2006 11:32'!
postCopy
	super postCopy.
	self classes: self classes copy! !

!MARelationDescription methodsFor: 'accessing-dynamic' stamp: 'lr 3/9/2006 11:32'!
reference
	"The reference within a MARelationDescription is calculated automatically from all the classes of the receiver, if set to nil. By setting the reference to a MAContainer instance it is possible to customize the reference description."

	^ super reference ifNil: [ self commonClass description ]! !

MARelationDescription subclass: #MAToManyRelationDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MAToManyRelationDescription commentStamp: '<historical>' prior: 0!
I am a description of an one-to-many relationship, possible values are instances of <code>Collection</code>.!

!MAToManyRelationDescription class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MAToManyRelationDescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
label
	^ '1:m Relation'! !

!MAToManyRelationDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitToManyRelationDescription: self! !

!MAToManyRelationDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
kind
	^ Collection! !

!MAToManyRelationDescription methodsFor: 'validating' stamp: 'lr 3/9/2006 11:32'!
validateKind: anObject
	super validateKind: anObject.
	anObject do: [ :object |
		(self classes anySatisfy: [ :class | object species = class ])
			ifFalse: [ MAKindError on: self ] ]! !

!MAToManyRelationDescription methodsFor: 'validating' stamp: 'lr 3/9/2006 11:32'!
validateRequired: anObject
	super validateRequired: anObject.
	(self isRequired and: [ anObject isCollection ] and: [ anObject isEmpty ])
		ifTrue: [ MARequiredError on: self ]! !

!MAToManyRelationDescription methodsFor: 'validating' stamp: 'lr 3/9/2006 11:32'!
validateSpecific: anObject
	super validateSpecific: anObject.
	anObject do: [ :each | self validate: each description: each description ]! !

MARelationDescription subclass: #MAToOneRelationDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MAToOneRelationDescription commentStamp: '<historical>' prior: 0!
I am a description of an one-to-one relationship.!

!MAToOneRelationDescription class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MAToOneRelationDescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
label
	^ '1:1 Relation'! !

!MAToOneRelationDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitToOneRelationDescription: self! !

!MAToOneRelationDescription methodsFor: 'validating' stamp: 'lr 3/9/2006 11:32'!
validateKind: anObject
	super validateKind: anObject.
	(self classes anySatisfy: [ :class | anObject species = class ])
		ifFalse: [ MAKindError on: self ]! !

!MAToOneRelationDescription methodsFor: 'validating' stamp: 'lr 3/9/2006 11:32'!
validateSpecific: anObject
	super validateSpecific: anObject.
	self validate: anObject description: anObject description! !

MAReferenceDescription subclass: #MATableDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MATableDescription commentStamp: '<historical>' prior: 0!
I am a description of tables, their cells and labels. I hold a reference to the description of my cells, that are all described using the same description. Possible values include instances of <code>*MATableModel*</code>.!

!MATableDescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:33'!
defaultColumnLabels
	^ OrderedCollection with: 'a' with: 'b' with: 'c'! !

!MATableDescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:33'!
defaultRowLabels
	^ OrderedCollection with: '1' with: '2' with: '3'! !

!MATableDescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:33'!
descriptionColumnLabels
	^ MAMemoDescription selector: #columnLabelsTextual label: 'Column Labels' priority: 250! !

!MATableDescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:33'!
descriptionDefault
	^ nil! !

!MATableDescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:33'!
descriptionRequired
	^ nil! !

!MATableDescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:33'!
descriptionRowLabels
	^ MAMemoDescription selector: #rowLabelsTextual label: 'Row Labels' priority: 250! !

!MATableDescription class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MATableDescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
label
	^ 'Table'! !

!MATableDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitTableDescription: self! !

!MATableDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
columnCount
	^ self columnLabels size! !

!MATableDescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
columnLabels
	^ self propertyAt: #columnLabels ifAbsent: [ self class defaultColumnLabels ]! !

!MATableDescription methodsFor: 'accessing-textual' stamp: 'lr 3/9/2006 11:32'!
columnLabelsTextual
	^ (MAStringDescription new toStringCollection: self columnLabels)
		asMultilineString! !

!MATableDescription methodsFor: 'accessing-textual' stamp: 'lr 3/9/2006 11:32'!
columnLabelsTextual: aString
	self columnLabels: (MAStringDescription new fromStringCollection: aString lines)! !

!MATableDescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
columnLabels: aCollection
	self propertyAt: #columnLabels put: aCollection! !

!MATableDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
kind
	^ MATableModel! !

!MATableDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
rowCount
	^ self rowLabels size! !

!MATableDescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
rowLabels
	^ self propertyAt: #rowLabels ifAbsent: [ self class defaultRowLabels ]! !

!MATableDescription methodsFor: 'accessing-textual' stamp: 'lr 3/9/2006 11:32'!
rowLabelsTextual
	^ (MAStringDescription new toStringCollection: self rowLabels)
		asMultilineString! !

!MATableDescription methodsFor: 'accessing-textual' stamp: 'lr 3/9/2006 11:32'!
rowLabelsTextual: aString
	self rowLabels: (MAStringDescription new fromStringCollection: aString lines)! !

!MATableDescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
rowLabels: aCollection
	self propertyAt: #rowLabels put: aCollection! !

!MATableDescription methodsFor: 'validation' stamp: 'lr 3/9/2006 11:32'!
validateSpecific: anObject
	super validateSpecific: anObject.
	(anObject rowCount ~= self rowCount
		or: [ anObject columnCount ~= self columnCount ])
			ifTrue: [ MAKindError on: self ].
	anObject contents do: [ :each |
		self validate: each description: self reference ]! !

MAReferenceDescription subclass: #MATokenDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MATokenDescription commentStamp: 'lr 2/12/2006 19:43' prior: 0!
I am a description of tokens all described by the referenced description, possible values are instances of <code>SequenceableCollection</code>.!

!MATokenDescription class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MATokenDescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
label
	^ 'Token'! !

!MATokenDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitTokenDescription: self! !

!MATokenDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
kind
	^ Array! !

MAElementDescription subclass: #MAStringDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MAStringDescription commentStamp: 'lr 2/12/2006 19:43' prior: 0!
I am a description of strings, possible values are instances of <code>String</code>. My visual representation could be a single line text-field. Use <code>*MAMemoDescription*</code> for multi-line strings.!

MAStringDescription subclass: #MAMemoDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MAMemoDescription commentStamp: 'lr 2/12/2006 19:43' prior: 0!
I am a description of multiline strings, possible values are instances of <code>String</code>. My visual representation could be a text-area field.!

!MAMemoDescription class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:33'!
defaultLineCount
	^ 3! !

!MAMemoDescription class methodsFor: 'accessing-description' stamp: 'lr 3/9/2006 11:33'!
descriptionLineCount
	^ (MANumberDescription selector: #lineCount label: 'Number of Lines' priority: 400 default: self defaultLineCount)
		beInteger; min: 1;
		yourself! !

!MAMemoDescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
label
	^ 'Memo'! !

!MAMemoDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitMemoDescription: self! !

!MAMemoDescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
lineCount
	^ self propertyAt: #lineCount ifAbsent: [ self class defaultLineCount ]! !

!MAMemoDescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
lineCount: anInteger
	^ self propertyAt: #lineCount put: anInteger! !

MAStringDescription subclass: #MAPasswordDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MAPasswordDescription commentStamp: 'lr 2/12/2006 19:44' prior: 0!
I am a description of a password string, possible values are instances of <code>String</code>. My visual representation could be a password field, where there are stars printed instead of the characters the user enters.!

!MAPasswordDescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
label
	^ 'Password'! !

!MAPasswordDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitPasswordDescription: self! !

!MAPasswordDescription methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isObfuscated: anObject
	^ anObject notNil
		and: [ anObject isString ]
		and: [ anObject allSatisfy: [ :each | each = $* ] ]! !

!MAPasswordDescription methodsFor: 'operators' stamp: 'lr 3/9/2006 11:32'!
obfuscated: anObject
	^ String new: (self toString: anObject) size withAll: $*! !

!MAStringDescription class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MAStringDescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
label
	^ 'String'! !

!MAStringDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitStringDescription: self! !

!MAStringDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
comparators
	^ Array
		with: (MAComparator
			selector: #includesSubString:
			descriptions: (Array with: (MAStringDescription new label: 'contains')))
		with: (MAComparator
			selector: #beginsWith:
			descriptions: (Array with: (MAStringDescription new label: 'begins with')))
		with: (MAComparator
			selector: #endsWith:
			descriptions: (Array with: (MAStringDescription new label: 'ends with')))
		with: (MAComparator
			selector: #beginsWith:
			descriptions: (Array with: (MAStringDescription new label: 'is')))! !

!MAStringDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
kind
	^ String! !

MAStringDescription subclass: #MASymbolDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MASymbolDescription commentStamp: '<historical>' prior: 0!
I am a description of symbols, possible values are instances of <code>Symbol</code>.!

!MASymbolDescription class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:33'!
label
	^ 'Symbol'! !

!MASymbolDescription methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor
	aVisitor visitSymbolDescription: self! !

!MASymbolDescription methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
kind
	^ Symbol! !

MAObject subclass: #MAMemento
	instanceVariableNames: 'model description'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Memento'!

MAMemento subclass: #MACachedMemento
	instanceVariableNames: 'cache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Memento'!

!MACachedMemento class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isAbstract
	^ false! !

!MACachedMemento methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
cache
	^ cache! !

!MACachedMemento methodsFor: 'actions' stamp: 'lr 3/9/2006 11:31'!
commit
	super commit.
	self push: self cache.
	self reset! !

!MACachedMemento methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
hasChanged
	^ self cache ~= self pullRaw! !

!MACachedMemento methodsFor: 'private' stamp: 'lr 3/9/2006 11:31'!
readUsing: aDescription
	^ self cache at: aDescription! !

!MACachedMemento methodsFor: 'actions' stamp: 'lr 3/9/2006 11:31'!
reset
	super reset.
	self setCache: self pull! !

!MACachedMemento methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:31'!
setCache: aDictionary
	cache := aDictionary! !

!MACachedMemento methodsFor: 'private' stamp: 'lr 3/9/2006 11:31'!
write: anObject using: aDescription
	self cache at: aDescription put: anObject! !

MACachedMemento subclass: #MACheckedMemento
	instanceVariableNames: 'original'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Memento'!

!MACheckedMemento methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
hasConflict
	^ self hasChanged and: [ self original ~= self pullRaw ]! !

!MACheckedMemento methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
original
	^ original! !

!MACheckedMemento methodsFor: 'actions' stamp: 'lr 3/9/2006 11:31'!
reset
	super reset.
	self setOriginal: self pullRaw! !

!MACheckedMemento methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:31'!
setOriginal: aDictionary
	original := aDictionary! !

!MACheckedMemento methodsFor: 'actions' stamp: 'lr 3/9/2006 11:31'!
validate
	self hasConflict
		ifFalse: [ ^ super validate ].
	self reset.
	MAConflictError on: self description! !

!MAMemento class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:33'!
model: aModel
	^ self model: aModel description: aModel description! !

!MAMemento class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:33'!
model: aModel description: aDescription
	^ self new
		setModel: aModel;
		setDescription: aDescription;
		reset; yourself! !

!MAMemento methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
commit
	"Commit the receiver into the model."! !

!MAMemento methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
description
	^ description! !

!MAMemento methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
model
	^ model! !

!MAMemento methodsFor: 'printing' stamp: 'lr 3/9/2006 11:32'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' model: '; print: self model! !

!MAMemento methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
pull
	"Utitlity method to pull the model into a dictionary mapping descriptions to values. nil values are replaced with the default ones of the model."

	| result |
	result := self pullRaw.
	result keysAndValuesDo: [ :key :value |
		value isNil
			ifTrue: [ result at: key put: key default yourself ] ].
	^ result! !

!MAMemento methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
pullRaw
	| result |
	result := Dictionary new.
	self description do: [ :each |
		result at: each put: (self model
			readUsing: each) ].
	^ result! !

!MAMemento methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
push: aDictionary
	"Utitlity method to push a dictionary mapping descriptions to values into the model."

	aDictionary keysAndValuesDo: [ :key :value |
		key isReadonly ifFalse: [
			self model
				write: value
				using: key ] ]! !

!MAMemento methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
reset
	"Reset the memento from the model."! !

!MAMemento methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:32'!
setDescription: aDescription
	description := aDescription! !

!MAMemento methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:32'!
setModel: aModel
	model := aModel! !

!MAMemento methodsFor: 'actions' stamp: 'lr 3/9/2006 11:32'!
validate
	"Check if the value is valid within the model. In case of problems an exception is raised."

	self description validate: self! !

MAMemento subclass: #MAStraitMemento
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Memento'!

!MAStraitMemento commentStamp: 'lr 9/22/2005 10:51' prior: 0!
The strait-memento forwards read- and write-access directly to the model.!

!MAStraitMemento class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ false! !

!MAStraitMemento methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
readUsing: aDescription
	^ (self model readUsing: aDescription)
		ifNil: [ aDescription default ]! !

!MAStraitMemento methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
write: anObject using: aDescription
	self model write: anObject using: aDescription! !

!MAObject class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:33'!
isAbstract
	^ true! !

!MAObject class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:33'!
new
	"Create a new instance of the receiving class and checks if it is concrete."

	self isAbstract
		ifTrue: [ self error: self name , ' is abstract.' ].
	^ super new! !

!MAObject class methodsFor: 'reflection' stamp: 'lr 3/9/2006 11:33'!
withAllConcreteClasses
	^ Array streamContents: [ :stream |
		self withAllConcreteClassesDo: [ :each |
			stream nextPut: each ] ]! !

!MAObject class methodsFor: 'reflection' stamp: 'lr 3/9/2006 11:33'!
withAllConcreteClassesDo: aBlock
	self withAllSubclassesDo: [ :each |
		each isAbstract ifFalse: [
			aBlock value: each ] ]! !

!MAObject methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
acceptMagritte: aVisitor! !

!MAObject methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
errorPropertyNotFound: aSelector
	self error: 'Property ' , aSelector , ' not found.'! !

!MAObject methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:32'!
hash
	"Answer a SmallInteger whose value is related to the receiver's identity. May be overridden, and should be overridden in any classes that define #=."

	^ self species hash! !

!MAObject methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
hasProperty: aKey
	"Test if the property aKey is defined within the receiver."

	^ self properties includesKey: aKey! !

!MAObject methodsFor: 'copying' stamp: 'lr 3/9/2006 11:32'!
postCopy
	"This method is called whenever a shallow copy of the receiver is made. Redefine this method in subclasses to copy other fields as necessary. Never forget to call super, else class invariants might be vialoated."

	super postCopy.
	properties := properties copy! !

!MAObject methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
properties
	"Answer the property dictionary of the receiver."

	^ properties ifNil: [ properties := Dictionary new ]! !

!MAObject methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
propertyAt: aKey
	"Answer the value of the property aKey, raises an error if the property doesn't exist."

	^ self propertyAt: aKey ifAbsent: [ self errorPropertyNotFound: aKey ]! !

!MAObject methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
propertyAt: aKey ifAbsentPut: aBlock
	"Answer the value of the property aKey, or if the property doesn't exist adds and answers the result of evaluating aBlock."

	^ self properties at: aKey ifAbsentPut: aBlock! !

!MAObject methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
propertyAt: aKey ifAbsent: aBlock
	"Answer the value of the property aKey, or the result of aBlock if the property doesn't exist."

	^ self properties at: aKey ifAbsent: aBlock! !

!MAObject methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
propertyAt: aKey put: aValue
	"Adds or replaces the property aKey with aValue."

	^ self properties at: aKey put: aValue! !

!MAObject methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:32'!
= anObject
	"Answer whether the receiver and the argument represent the same object. This default implementation checks if the species of the compared objects are the same, so that superclasses might call super before performing their own check. Also redefine the message #hash when redefining this message."

	^ self species = anObject species! !

Object subclass: #MAScaffolder
	instanceVariableNames: 'description values'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Models'!

!MAScaffolder class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:33'!
description: aDescription
	^ self new
		description: aDescription;
		yourself! !

!MAScaffolder methodsFor: 'accessing-configuration' stamp: 'lr 3/9/2006 11:32'!
defaultDescription
	^ MAContainer new! !

!MAScaffolder methodsFor: 'accessing-configuration' stamp: 'lr 3/9/2006 11:32'!
defaultDictionary
	^ Dictionary new! !

!MAScaffolder methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
description
	^ description! !

!MAScaffolder methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
description: aDescription
	description := aDescription! !

!MAScaffolder methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:32'!
initialize
	super initialize.
	self description: self defaultDescription.
	self values: self defaultDictionary! !

!MAScaffolder methodsFor: 'model' stamp: 'lr 3/9/2006 11:32'!
readUsing: aDescription
	^ self values at: aDescription ifAbsent: [ nil ]! !

!MAScaffolder methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
values
	^ values! !

!MAScaffolder methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
values: aDictionary
	values := aDictionary! !

!MAScaffolder methodsFor: 'model' stamp: 'lr 3/9/2006 11:32'!
write: anObject using: aDescription
	self values at: aDescription put: anObject! !

Object subclass: #MATableModel
	instanceVariableNames: 'rowCount columnCount contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Models'!

!MATableModel commentStamp: 'lr 2/26/2006 10:02' prior: 0!
I am a model class representing a table within the Magritte framework. Internally I store the my cells within a flat array, however users access my data giving <emph>row</emph> and <emph>column</emph> coordinates with <code>#at:at:</code> and <code>#at:at:put:</code>. I can support reshaping myself, but of course this might lead to loss of data-cells.!

!MATableModel class methodsFor: 'instance-creation' stamp: 'lr 3/9/2006 11:33'!
rows: aRowCount columns: aColumnCount
	^ self rows: aRowCount columns: aColumnCount contents: (Array new: aRowCount * aColumnCount)! !

!MATableModel class methodsFor: 'instance-creation' stamp: 'lr 3/9/2006 11:33'!
rows: aRowCount columns: aColumnCount contents: anArray
	^ self new
		setRowCount: aRowCount;
		setColumnCount: aColumnCount;
		setContents: anArray;
		yourself! !

!MATableModel methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
at: aRowIndex at: aColumnIndex
	"Answer the contents of <code>aRowIndex</code> and <code>aColumnIndex</code>. Raises an error if the coordinates are out of bounds."

	self checkAt: aRowIndex at: aColumnIndex.
	^ self uncheckedAt: aRowIndex at: aColumnIndex! !

!MATableModel methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
at: aRowIndex at: aColumnIndex put: aValue
	"Set the contents of <code>aRowIndex</code> and <code>aColumnIndex</code> to aValue. Raises an error if the coordinates are out of bounds."

	self checkAt: aRowIndex at: aColumnIndex.
	^ self uncheckedAt: aRowIndex at: aColumnIndex put: aValue! !

!MATableModel methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
checkAt: aRowIndex at: aColumnIndex
	(aRowIndex between: 1 and: self rowCount)
		ifFalse: [ self error: 'Row subscript out of range.' ].
	(aColumnIndex between: 1 and: self columnCount)
		ifFalse: [ self error: 'Column subscript out of range.' ]! !

!MATableModel methodsFor: 'enumeration' stamp: 'lr 3/9/2006 11:32'!
collect: aBlock
	^ self copyEmpty in: [ :table |
		self do: [ :row :col :val |
			table
				at: row at: col
				put: (aBlock value: row value: col value: val) ].
		table ]! !

!MATableModel methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
columnCount
	"Answer the column count of the table."

	^ columnCount! !

!MATableModel methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
contents
	^ contents! !

!MATableModel methodsFor: 'copying' stamp: 'lr 3/9/2006 11:32'!
copyEmpty
	^ self class rows: self rowCount columns: self columnCount! !

!MATableModel methodsFor: 'copying' stamp: 'lr 3/9/2006 11:32'!
copyRows: aRowCount columns: aColumnCount
	| table |
	table := self class rows: aRowCount columns: aColumnCount.
	1 to: (self rowCount min: aRowCount) do: [ :row |
		1 to: (self columnCount min: aColumnCount) do: [ :col |
			table
				uncheckedAt: row at: col
				put: (self uncheckedAt: row at: col) ] ].
	^ table! !

!MATableModel methodsFor: 'enumeration' stamp: 'lr 3/9/2006 11:32'!
do: aBlock
	1 to: self rowCount do: [ :row |
		1 to: self columnCount do: [ :col |
			aBlock
				value: row value: col
				value: (self uncheckedAt: row at: col) ] ]! !

!MATableModel methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:32'!
hash
	^ self contents hash! !

!MATableModel methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
indexAt: aRowIndex at: aColumnIndex
	^ (aRowIndex - 1) * self columnCount + aColumnIndex! !

!MATableModel methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
pointAt: anIndex
	^ Point x: (anIndex - 1 // self columnCount + 1) y: (anIndex - 1 \\ self columnCount + 1)! !

!MATableModel methodsFor: 'copying' stamp: 'lr 3/9/2006 11:32'!
postCopy
	super postCopy.
	self setContents: self contents copy! !

!MATableModel methodsFor: 'operations' stamp: 'lr 3/9/2006 11:32'!
reshapeRows: aRowCount columns: aColumnCount
	"Changes the size of the receiving table to <code>aRowCount</code> times <code>aColumnCount</code>, throwing away elemnts that are cut off and initializing empty cells with <code>nil</code>."

	self setContents: (self copyRows: aRowCount columns: aColumnCount) contents.
	self setRowCount: aRowCount; setColumnCount: aColumnCount! !

!MATableModel methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
rowCount
	"Answer the row count of the table."

	^ rowCount! !

!MATableModel methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:32'!
setColumnCount: anInteger
	columnCount := anInteger! !

!MATableModel methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:32'!
setContents: anArray
	contents := anArray! !

!MATableModel methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:32'!
setRowCount: anInteger
	rowCount := anInteger! !

!MATableModel methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
uncheckedAt: aRowIndex at: aColumnIndex
	^ self contents at: (self indexAt: aRowIndex at: aColumnIndex)! !

!MATableModel methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
uncheckedAt: aRowIndex at: aColumnIndex put: aValue
	^ self contents at: (self indexAt: aRowIndex at: aColumnIndex) put: aValue! !

!MATableModel methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:32'!
= aTable
	^ self species = aTable species
		and: [ self rowCount = aTable rowCount ]
		and: [ self columnCount = aTable columnCount ]
		and: [ self contents = aTable contents ]! !

Object subclass: #MAVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Visitor'!

!MAVisitor commentStamp: 'lr 9/12/2005 12:14' prior: 0!
I am a visitor responsible to visit Magritte descriptions. I am an abstract calss providing a default implementation for concrete visitors. The protocol I am implementing reflects the hierarchy of MADescription with its subclasses so that visiting a specific class automatically calls less specific implementations in case the specific implementation has been left out. This code was automatically using #buildVisitorHierarchyForClass:selector:classified: on my class-side.!

MAVisitor subclass: #MAStreamingVisitor
	instanceVariableNames: 'stream object'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Visitor'!

MAStreamingVisitor subclass: #MAReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Visitor'!

MAReader subclass: #MABinaryReader
	instanceVariableNames: 'version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Visitor'!

!MABinaryReader class methodsFor: 'instance-creation' stamp: 'lr 3/9/2006 11:32'!
read: aStream
	^ self new read: aStream! !

!MABinaryReader methodsFor: 'private' stamp: 'lr 3/9/2006 11:31'!
next
	self version: self stream next.
	self version isInteger
		ifFalse: [ self error: 'Invalid File Format.' ].
	self stream next in: [ :description |
		description isDescription
			ifFalse: [ self error: 'Invalid File Description.' ].
		description class version <= self version
			ifFalse: [ self error: 'Invalid File Version.' ].
		self visit: description ].
	^ self object! !

!MABinaryReader methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:31'!
read: aStream
	^ self stream: (MACompatibility referenceStream: aStream); next! !

!MABinaryReader methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
version
	^ version! !

!MABinaryReader methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
version: aVersion
	version := aVersion! !

!MABinaryReader methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:31'!
visitContainer: aDescription
	self object: (self stream next
		ifNotNilDo: [ :class | class new ]).
	self object isNil
		ifTrue: [ ^ self ].
	aDescription do: [ :each |
		each isPersistent
			ifTrue: [ self next ] ]! !

!MABinaryReader methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:31'!
visitElementDescription: aDescription
	self object write: self stream next using: aDescription! !

!MABinaryReader methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:31'!
visitToOneRelationDescription: aDescription
	| parent |
	parent := self object.
	[ parent write: self next using: aDescription ]
		ensure: [ self object: parent ]! !

!MAReader class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:33'!
read: aStream description: aDescription
	^ self new read: aStream description: aDescription! !

!MAReader methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
error: aString
	MAReadError signal: aString! !

!MAReader methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
read: aStream description: aDescription
	self
		stream: aStream;
		visit: aDescription.
	^ self object! !

MAReader subclass: #MAStringReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Visitor'!

!MAStringReader methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
read: aStream description: aDescription
	^ aStream atEnd
		ifFalse: [ super read: aStream description: aDescription ]! !

!MAStringReader methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitBooleanDescription: aDescription
	(aDescription trueString = self contents
		or: [ aDescription trueStrings includes: self contents ])
			ifTrue: [ ^ self object: true ].
	(aDescription falseString = self contents
		or: [ aDescription falseStrings includes: self contents ])
			ifTrue: [ ^ self object: false ]! !

!MAStringReader methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitClassDescription: aDescription
	self shouldNotImplement! !

!MAStringReader methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitColorDescription: aDescription
	self object: (aDescription kind fromString: self contents)! !

!MAStringReader methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitContainer: anObject
	self shouldNotImplement! !

!MAStringReader methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitElementDescription: aDescription
	"This implementation can be very dangerous and might lead to a potential security hole, since the default implementation of #readFrom: in Object evaluates the expression to find its value. Most subclasses like Number, Date, Time, ... override this implementation, but some others (like Boolean) do not."

	self object: (aDescription kind readFrom: self stream)! !

!MAStringReader methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitFileDescription: aDescription
	self shouldNotImplement! !

!MAStringReader methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitMultipleOptionDescription: aDescription
	self object: (Array streamContents: [ :output |
		[ self stream atEnd ] whileFalse: [
			output nextPut: (aDescription reference
				fromString: (self stream upToAll: ', ')) ] ])! !

!MAStringReader methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitRelationDescription: aDescription
	self shouldNotImplement! !

!MAStringReader methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitSingleOptionDescription: aDescription
	self visit: aDescription reference! !

!MAStringReader methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitStringDescription: aDescription
	self object: self contents! !

!MAStringReader methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitSymbolDescription: aDescription
	self object: self contents asSymbol! !

!MAStringReader methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitTableDescription: aDescription
	self shouldNotImplement! !

!MAStringReader methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitTokenDescription: aDescription
	self object: (aDescription kind streamContents: [ :output |
		[ self stream atEnd ] whileFalse: [
			output nextPut: (aDescription reference
				fromString: (self stream upTo: $ )) ] ])! !

!MAStreamingVisitor methodsFor: 'streaming' stamp: 'lr 3/9/2006 11:32'!
contents
	^ self stream contents! !

!MAStreamingVisitor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
object
	^ object! !

!MAStreamingVisitor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
object: anObject
	object := anObject! !

!MAStreamingVisitor methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
object: anObject during: aBlock
	| previous |
	previous := self object. self object: anObject.
	aBlock ensure: [ self object: previous ]! !

!MAStreamingVisitor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
stream
	^ stream! !

!MAStreamingVisitor methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
stream: aStream
	stream := aStream! !

MAStreamingVisitor subclass: #MAWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Visitor'!

MAWriter subclass: #MABinaryWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Visitor'!

!MABinaryWriter methodsFor: 'private' stamp: 'lr 3/9/2006 11:31'!
defaultWriteStream
	^ MACompatibility referenceStream: ByteArray new writeStream! !

!MABinaryWriter methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:31'!
visitContainer: aDescription
	super visitContainer: aDescription.
	self object isNil
		ifTrue: [ ^ self stream nextPut: nil ].
	self stream nextPut: self object class.
	aDescription do: [ :each |
		each isPersistent ifTrue: [
			self
				object: (self object readUsing: each)
				during: [ self visit: each ] ] ]! !

!MABinaryWriter methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:31'!
visitDescription: aDescription
	self stream nextPut: aDescription class version; nextPut: aDescription! !

!MABinaryWriter methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:31'!
visitElementDescription: aDescription
	super visitElementDescription: aDescription.
	self stream nextPut: self object! !

!MABinaryWriter methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:31'!
visitToOneRelationDescription: aDescription
	self visitDescription: aDescription.
	self visit: self object description! !

MAWriter subclass: #MAStringWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Visitor'!

!MAStringWriter methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
defaultWriteStream
	^ String new writeStream! !

!MAStringWriter methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitBooleanDescription: aDescription
	self stream nextPutAll: (self object
		ifTrue: [ aDescription trueString ]
		ifFalse: [ aDescription falseString ])! !

!MAStringWriter methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitClassDescription: aDescription
	self stream nextPutAll: self object label! !

!MAStringWriter methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitColorDescription: aDescription
	stream nextPut: $#.
	stream nextPutAll: ((255 * self object red) asInteger
		printPaddedWith: $0 to: 2 base: 16).
	stream nextPutAll: ((255 * self object green) asInteger
		printPaddedWith: $0 to: 2 base: 16).
	stream nextPutAll: ((255 * self object blue) asInteger
		printPaddedWith: $0 to: 2 base: 16)! !

!MAStringWriter methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitContainer: aDescription
	aDescription do: [ :each |
		each isVisible ifTrue: [
			each stringWriter
				write: (self object readUsing: each)
				description: each to: stream.
			^ self ] ]! !

!MAStringWriter methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitElementDescription: aDescription
	self stream nextPutAll: self object asString! !

!MAStringWriter methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitFileDescription: aDescription
	self stream nextPutAll: self object filename! !

!MAStringWriter methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitMultipleOptionDescription: aDescription
	self object
		do: [ :each | self object: each during: [ self visit: aDescription reference ] ]
		separatedBy: [ self stream nextPutAll: ', ' ]! !

!MAStringWriter methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitSingleOptionDescription: aDescription
	self visit: aDescription reference! !

!MAStringWriter methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitStringDescription: aDescription
	self stream nextPutAll: self object! !

!MAStringWriter methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitTimeDescription: aDescription
	self object print24: true showSeconds: true on: self stream! !

!MAStringWriter methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitTimeStampDescription: aDescription
	self stream print: self object asDate; space.
	self stream print: self object asTime! !

!MAStringWriter methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitTokenDescription: aDescription
	self object
		do: [ :each | self object: each during: [ self visit: aDescription reference ] ]
		separatedBy: [ self stream nextPutAll: ' '  ]! !

!MAStringWriter methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitToManyRelationDescription: aDescription
	self object
		do: [ :each | self object: each during: [ self visit: each description ] ]
		separatedBy: [ self stream nextPutAll: ', ' ]! !

!MAStringWriter methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitToOneRelationDescription: aDescription
	self visit: self object description! !

!MAStringWriter methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
write: anObject description: aDescription to: aStream
	anObject isNil ifTrue: [ ^ aDescription undefined ].
	^ super write: anObject description: aDescription to: aStream! !

!MAWriter class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:33'!
write: anObject
	^ self new write: anObject! !

!MAWriter class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:33'!
write: anObject description: aDescription
	^ self new write: anObject description: aDescription! !

!MAWriter class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:33'!
write: anObject description: aDescription to: aStream
	^ self new write: anObject description: aDescription to: aStream! !

!MAWriter methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
defaultWriteStream
	self subclassResponsibility! !

!MAWriter methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
error: aString
	MAWriteError signal: aString! !

!MAWriter methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
write: anObject
	^ self write: anObject description: anObject description! !

!MAWriter methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
write: anObject description: aDescription
	^ self write: anObject description: aDescription to: self defaultWriteStream! !

!MAWriter methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
write: anObject description: aDescription to: aStream
	self
		object: anObject;
		stream: aStream;
		visit: aDescription.
	^ self contents! !

!MAVisitor class methodsFor: 'private' stamp: 'lr 3/9/2006 11:33'!
buildVisitorHierarchyForClass: aClass selector: aBlock classified: aSelector
	" self buildVisitorHierarchyForClass: MADescription selector: [ :class | 'visit' , (class name allButFirst: 2) , ':' ] classified: #'visiting-description' "

	aClass withAllSubclassesDo: [ :class |
		(class category beginsWith: 'Magritte') ifTrue: [
			self
				compile: (String streamContents: [ :stream |
					stream nextPutAll: (aBlock value: class); nextPutAll: ' anObject'; cr.
					class = aClass ifFalse: [
						stream tab; nextPutAll: 'self '; nextPutAll: (aBlock value: class superclass); nextPutAll: ' anObject.' ] ])
				classified: aSelector.
			class
				compile: (String streamContents: [ :stream |
					stream nextPutAll: 'acceptMagritte: aVisitor'; cr.
					stream tab; nextPutAll: 'aVisitor '; nextPutAll: (aBlock value: class); nextPutAll: ' self.' ])
				classified: #visiting ] ]! !

!MAVisitor methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
visitAll: aCollection
	aCollection do: [ :each | self visit: each ]! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitBooleanDescription: anObject
	self visitElementDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitClassDescription: anObject
	self visitElementDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitColorDescription: anObject
	self visitElementDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitContainer: anObject
	self visitDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitDateDescription: anObject
	self visitMagnitudeDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitDurationDescription: anObject
	self visitMagnitudeDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitElementDescription: anObject
	self visitDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitFileDescription: anObject
	self visitElementDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitMagnitudeDescription: anObject
	self visitElementDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitMemoDescription: anObject
	self visitStringDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitMultipleOptionDescription: anObject
	self visitOptionDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitNumberDescription: anObject
	self visitMagnitudeDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitOptionDescription: anObject
	self visitReferenceDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitPasswordDescription: anObject
	self visitStringDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitPriorityContainer: anObject
	self visitContainer: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitReferenceDescription: anObject
	self visitElementDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitRelationDescription: anObject
	self visitReferenceDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitReportContainer: anObject
	self visitContainer: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitSingleOptionDescription: anObject
	self visitOptionDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitStringDescription: anObject
	self visitElementDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitSymbolDescription: anObject
	self visitStringDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitTableDescription: anObject
	self visitReferenceDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitTableReference: anObject
	^ self visitReferenceDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitTimeDescription: anObject
	self visitMagnitudeDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitTimeStampDescription: anObject
	self visitMagnitudeDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitTokenDescription: anObject
	self visitReferenceDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitToManyRelationDescription: anObject
	self visitRelationDescription: anObject! !

!MAVisitor methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitToOneRelationDescription: anObject
	self visitRelationDescription: anObject! !

!MAVisitor methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:32'!
visit: anObject
	anObject acceptMagritte: self! !

!Object class methodsFor: '*magritte-model-accessing' stamp: 'lr 3/27/2006 15:47'!
description
	^ MADescriptionBuilder for: self! !

!Object methodsFor: '*magritte-model-converting' stamp: 'lr 3/9/2006 11:31'!
asCondition
	^ self! !

!Object methodsFor: '*magritte-model-accessing' stamp: 'lr 3/9/2006 11:31'!
description
	"Return the description of the reciever. Subclasses might override this message to return instance-based descriptions."

	^ self class description! !

!Object methodsFor: '*magritte-model-testing' stamp: 'lr 3/9/2006 11:31'!
isAccessor
	^ false! !

!Object methodsFor: '*magritte-model-testing' stamp: 'lr 3/9/2006 11:31'!
isCondition
	^ false! !

!Object methodsFor: '*magritte-model-testing' stamp: 'lr 3/9/2006 11:31'!
isDescription
	^ false! !

!Object methodsFor: '*magritte-model-accessing' stamp: 'lr 3/9/2006 11:31'!
mementoClass
	"Return a class to be used to remember or cache the receiver, namely a memento object."

	^ MACheckedMemento! !

!Object methodsFor: '*magritte-model-accessing' stamp: 'lr 3/9/2006 11:31'!
queryString
	"Answer a computer readable string to be used for full text searches on the receiver. Subclasses might want to refine the default string that is built based on the descriptions of the reciever."

	^ String streamContents: [ :stream |
		self description
			do: [ :each | stream nextPutAll: (each toString: (self readUsing: each)) ]
			separatedBy: [ stream cr ] ]! !

!Object methodsFor: '*magritte-model-model' stamp: 'lr 3/9/2006 11:31'!
readUsing: aDescription
	"Dispatch the read-access to the receiver using the accessor of aDescription."

	^ aDescription accessor read: self! !

!Object methodsFor: '*magritte-model-model' stamp: 'lr 3/9/2006 11:31'!
write: anObject using: aDescription
	"Dispatch the write-access to the receiver of anObject using the accessor of aDescription."

	aDescription accessor write: anObject to: self! !

Error subclass: #MAError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Exception'!

MAError subclass: #MAReadError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Exception'!

MAError subclass: #MAValidationError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Exception'!

MAValidationError subclass: #MAConditionError
	instanceVariableNames: 'explanation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Exception'!

!MAConditionError class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:32'!
on: aDescription explanation: aString
	^ self new
		setDescription: aDescription;
		setExplanation: aString;
		signal! !

!MAConditionError methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
explanation
	^ explanation! !

!MAConditionError methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:32'!
setExplanation: aString
	explanation := aString! !

MAValidationError subclass: #MAConflictError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Exception'!

!MAConflictError methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
explanation
	^ 'Input is conflicting with concurrent modification'! !

MAValidationError subclass: #MAKindError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Exception'!

!MAKindError methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
explanation
	^ 'Invalid input given'! !

MAValidationError subclass: #MAMultipleErrors
	instanceVariableNames: 'collection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Exception'!

!MAMultipleErrors class methodsFor: 'instance-creation' stamp: 'lr 3/9/2006 11:33'!
on: aDescription errors: aCollection
	^ self new
		setDescription: aDescription;
		setCollection: aCollection;
		signal! !

!MAMultipleErrors methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
collection
	^ collection! !

!MAMultipleErrors methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
explanation
	^ 'Multiple errors'! !

!MAMultipleErrors methodsFor: 'printing' stamp: 'lr 3/9/2006 11:32'!
printOn: aStream
	self collection
		do: [ :each | aStream print: each ]
		separatedBy: [ aStream nextPut: Character cr ]! !

!MAMultipleErrors methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:32'!
setCollection: aCollection
	collection := aCollection! !

MAValidationError subclass: #MARangeError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Exception'!

!MARangeError methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
explanation
	| min max |
	min := self tag toString: self tag min.
	max := self tag toString: self tag max.
	^ String streamContents: [ :stream |
		stream nextPutAll: 'Input must be '.
		self tag max isNil
			ifTrue: [ stream nextPutAll: 'above '; nextPutAll: min ]
			ifFalse: [
				self tag min isNil
					ifTrue: [ stream nextPutAll: 'below '; nextPutAll: max ]
					ifFalse: [ stream nextPutAll: 'between '; nextPutAll: min; nextPutAll: ' and '; nextPutAll: max ] ] ]! !

MAValidationError subclass: #MARequiredError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Exception'!

!MARequiredError methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
explanation
	^ 'Input is required but no input given'! !

!MAValidationError class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:33'!
on: aDescription
	^ self new
		setDescription: aDescription;
		signal! !

!MAValidationError methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
explanation
	self subclassResponsibility! !

!MAValidationError methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
hasLabel
	^ self tag hasLabel! !

!MAValidationError methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
label
	^ self tag label ifNil: [ self tag asString ]! !

!MAValidationError methodsFor: 'printing' stamp: 'lr 3/9/2006 11:32'!
printOn: aStream
	aStream nextPutAll: self label; nextPutAll: ': '; nextPutAll: self explanation! !

!MAValidationError methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:32'!
setDescription: aDescription
	self tag: aDescription! !

MAError subclass: #MAWriteError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Exception'!

!SequenceableCollection methodsFor: '*magritte-model' stamp: 'lr 3/9/2006 11:31'!
copyWithAll: aCollection
	^ (self species new: self size + aCollection size)
		replaceFrom: 1 to: self size with: self startingAt: 1;
		replaceFrom: self size + 1 to: self size + aCollection size with: aCollection startingAt: 1;
		yourself! !

!SequenceableCollection methodsFor: '*magritte-model' stamp: 'lr 3/9/2006 11:31'!
moveDown: anObject
	| first second |
	first := self indexOf: anObject ifAbsent: [ ^ 0 ].
	second := first < self size ifTrue: [ first + 1 ] ifFalse: [ ^ first ].
	self swap: first with: second.
	^ second! !

!SequenceableCollection methodsFor: '*magritte-model' stamp: 'lr 3/9/2006 11:31'!
moveUp: anObject
	| first second |
	first := self indexOf: anObject ifAbsent: [ ^ 0 ].
	second := first > 1 ifTrue: [ first - 1 ] ifFalse: [ ^ first ].
	self swap: first with: second.
	^ second! !

!SequenceableCollection methodsFor: '*magritte-model' stamp: 'lr 3/9/2006 11:31'!
reduce: aBlock
	| result |
	result := self first.
	2 to: self size do: [ :index |
		result := aBlock
			value: result
			value: (self at: index) ].
	^ result! !

!Class methodsFor: '*magritte-model-configuration' stamp: 'lr 3/9/2006 11:31'!
defaultContainer
	"Return the default description container."

	^ MAPriorityContainer new
		label: self label;
		yourself! !

!Class methodsFor: '*magritte-model-accessing' stamp: 'lr 3/9/2006 11:31'!
label
	"Answer a human-readable name of the receiving class. This implementation tries to be smart and return a nice label, unfortunately for a lot of classes this doesn't work well so subclasses might want to override this method and return soemthing more meaningfull to end-users."

	| start input |
	start := self name findFirst: [ :each | each isLowercase ].
	input := (self name copyFrom: (1 max: start - 1) to: self name size)
		readStream.
	^ String streamContents: [ :stream |
		[ input atEnd ] whileFalse: [
			stream nextPut: input next.
			(input atEnd or: [ input peek isLowercase ])
				ifFalse: [ stream nextPut: Character space ] ] ]! !

ProtoObject subclass: #MAConditionBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Condition'!

!MAConditionBuilder methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
doesNotUnderstand: aMessage
	^ MASelectorCondition
		selector: aMessage selector
		arguments: (aMessage arguments
			collect: #asCondition)! !

!MAConditionBuilder methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isNil
	^ MASelectorCondition selector: #isNil arguments: #()! !

ProtoObject subclass: #MAProxyObject
	instanceVariableNames: 'realSubject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Utility'!

MAProxyObject subclass: #MADynamicObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Utility'!

!MADynamicObject commentStamp: 'lr 12/23/2005 11:42' prior: 0!
A dynamic object can be used for almost any property within Magritte that is not static but calculated dynamically. This is a shortcut to avoid having to build context sensitive descriptions manually over and over again, however there are a few drawbacks: 

- Some messages sent to this proxy, e.g. #class and #value, might not get resolved properly.
- Raising an unhandled exception will not open a debugger on your proxy, because the debugger is unable to properly work with the invalid object and might even crash your image. In this rare case nil is returned, what will probably cause a different error later on.!

!MADynamicObject methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
realSubject
	^ super realSubject on: UnhandledError do: [ :err | nil ]! !

!MAProxyObject class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:33'!
on: anObject
	^ self basicNew realSubject: anObject! !

!MAProxyObject methodsFor: 'copying' stamp: 'lr 3/9/2006 11:32'!
copy
	"It doesn't make sense to copy proxies in most cases, the real-subject needs still to be looked up and will probably return a new instance on every call anyway."

	^ self! !

!MAProxyObject methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
doesNotUnderstand: aMessage
	^ self realSubject
		perform: aMessage selector
		withArguments: aMessage arguments! !

!MAProxyObject methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isMorph
	"Answer false, since I am no morph. Squeak is calling this method after image-startup and might lock if I do not answer to this message."

	^ false! !

!MAProxyObject methodsFor: 'testing' stamp: 'lr 3/9/2006 11:32'!
isNil
	"This method is required to properly return true if the realSubject is nil."

	^ self realSubject isNil! !

!MAProxyObject methodsFor: 'printing' stamp: 'lr 3/9/2006 11:32'!
printOn: aStream
	"Print the receiver on aStream but within square-brackets to show that it is a proxied instance."

	aStream nextPut: $[; print: self realSubject; nextPut: $]! !

!MAProxyObject methodsFor: 'printing' stamp: 'lr 3/9/2006 11:32'!
printString
	^ String streamContents: [ :stream | self printOn: stream ]! !

!MAProxyObject methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
realSubject
	^ realSubject! !

!MAProxyObject methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
realSubject: anObject
	realSubject := anObject! !

!UndefinedObject methodsFor: '*magritte-model-accessing' stamp: 'lr 3/9/2006 11:31'!
label
	^ 'n/a'! !

!String methodsFor: '*magritte-model-accessing' stamp: 'lr 3/9/2006 11:31'!
lines
	"Retrun a collection with the string-lines of the receiver."

	^ Array streamContents: [ :output |
		self readStream in: [ :input |
			[ input atEnd ] whileFalse: [
				output nextPut: (input upTo: Character cr).
				input peek = Character lf
					ifTrue: [ input next ] ] ] ]! !

!String methodsFor: '*magritte-model-testing' stamp: 'lr 3/9/2006 11:31'!
matches: aString
	aString isEmpty ifTrue: [ ^ true ].
	^ (aString includesAnyOf: '*#')
		ifTrue: [ aString match: self ]
		ifFalse: [ self includesSubstring: aString caseSensitive: false ]! !

!Collection methodsFor: '*magritte-model' stamp: 'lr 3/9/2006 11:31'!
asMultilineString
	^ String streamContents: [ :stream |
		self
			do: [ :each | stream nextPutAll: each ]
			separatedBy: [ stream nextPut: Character cr ] ]! !

!Collection methodsFor: '*magritte-model' stamp: 'lr 3/9/2006 11:31'!
copyWithAll: aCollection
	^ self copy
		addAll: aCollection;
		yourself! !

!Collection methodsFor: '*magritte-model' stamp: 'lr 3/9/2006 11:31'!
copyWithoutFirst: anObject
	| done |
	done := false.
	^ self reject: [ :each |
		each = anObject
			and: [ done not ]
			and: [ done := true ] ]! !

!Integer methodsFor: '*magritte-model-converting' stamp: 'lr 3/9/2006 11:31'!
asFileSize
	#( 'B' 'KB' 'MB' 'GB' 'TB' 'PB' 'EB' 'ZB' 'YB' )
		inject: self
		into: [ :value :each |
			value < 1024
				ifFalse: [ value // 1024 ]
				ifTrue: [ ^ value asString , ' ' , each ] ]! !

!BlockContext methodsFor: '*magritte-model-converting' stamp: 'lr 3/9/2006 11:31'!
asCondition
	"Convert the receiver to a condition, e.g. a parse tree representing the condition within the block. This is done by passing a proxy into the block and manually building a tree of conditions. This doesn't work for all code, but has the big advantage to make it fully serializeable."

	^ self value: MAConditionBuilder new! !
MADescriptionBuilder initialize!
MAFileModel initialize!
